@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -40,6 +40,7 @@ from linotp.lib.base import BaseController
 from pylons.templating import render_mako as render
 from mako.exceptions import CompileException
 
+from paste.deploy.converters import asbool
 
 # Our Token stuff
 from linotp.lib.token   import TokenIterator
@@ -58,6 +59,7 @@ from linotp.lib.reply   import sendError
 
 from linotp.lib.util    import remove_empty_lines
 from linotp.lib.util import get_client
+from linotp.lib.util import unicode_compare
 from linotp.model.meta import Session
 
 from linotp.lib.policy import checkPolicyPre, PolicyException, getAdminPolicies, getPolicyDefinitions
@@ -66,7 +68,6 @@ from pylons.i18n.translation import _
 
 audit = config.get('audit')
 
-import traceback
 import logging
 
 log = logging.getLogger(__name__)
@@ -90,7 +91,6 @@ class ManageController(BaseController):
             audit.initialize()
             c.audit['success'] = False
             c.audit['client'] = get_client()
-            self.set_language()
 
             c.version = get_version()
             c.licenseinfo = get_copyright_info()
@@ -113,8 +113,7 @@ class ManageController(BaseController):
                 check_session()
 
         except Exception as exx:
-            log.error("[__before__::%r] exception %r" % (action, exx))
-            log.error("[__before__] %s" % traceback.format_exc())
+            log.exception("[__before__::%r] exception %r" % (action, exx))
             Session.rollback()
             Session.close()
             return sendError(response, exx, context='before')
@@ -140,6 +139,7 @@ class ManageController(BaseController):
         '''
 
         try:
+            c.debug = asbool(config.get('debug', False))
             c.title = "LinOTP Management"
             admin_user = getUserFromRequest(request)
             if admin_user.has_key('login'):
@@ -203,18 +203,16 @@ class ManageController(BaseController):
             c.logout_url = "%s://log-me-out:fake@%s/manage/logout" % (url_scheme, http_host)
 
             Session.commit()
-            ren = render('/manage/start.mako')
+            ren = render('/manage/manage-base.mako')
             return ren
 
         except PolicyException as pe:
-            log.error("[index] Error during checking policies: %r" % pe)
-            log.error("[index] %s" % traceback.format_exc())
+            log.exception("[index] Error during checking policies: %r" % pe)
             Session.rollback()
             return sendError(response, unicode(pe), 1)
 
         except Exception as ex:
-            log.error("[index] failed! %r" % ex)
-            log.error("[index] %s" % traceback.format_exc())
+            log.exception("[index] failed! %r" % ex)
             Session.rollback()
             return sendError(response, ex)
 
@@ -375,7 +373,7 @@ class ManageController(BaseController):
                             tok['LinOtp.CountWindow'],
                             tok['LinOtp.SyncWindow'],
                             tok['LinOtp.Userid'],
-                            tok['LinOtp.IdResolver'], ]
+                            tok['LinOtp.IdResClass'], ]
                     }
                     )
 
@@ -391,14 +389,12 @@ class ManageController(BaseController):
             return json.dumps(res, indent=3)
 
         except PolicyException as pe:
-            log.error("[tokenview_flexi] Error during checking policies: %r" % pe)
-            log.error("[tokenview_flexi] %s" % traceback.format_exc())
+            log.exception("[tokenview_flexi] Error during checking policies: %r" % pe)
             Session.rollback()
             return sendError(response, unicode(pe), 1)
 
         except Exception as e:
-            log.error("[tokenview_flexi] failed: %r" % e)
-            log.error("[tokenview_flexi] %s" % traceback.format_exc())
+            log.exception("[tokenview_flexi] failed: %r" % e)
             Session.rollback()
             return sendError(response, e)
 
@@ -428,7 +424,8 @@ class ManageController(BaseController):
             user = getUserFromParam(param, optional)
             # check admin authorization
             # check if we got a realm or resolver, that is ok!
-            checkPolicyPre('admin', 'userlist', { 'user': "dummy", 'realm' : c.realm })
+            checkPolicyPre('admin', 'userlist', { 'user': user.login,
+                                                 'realm' : c.realm })
 
             if c.filter == "":
                 c.filter = "*"
@@ -476,7 +473,11 @@ class ManageController(BaseController):
                     'mobile' :5, 'phone' : 6, 'userid' : 7 }
             if c.dir == "desc":
                 reverse = True
-            lines = sorted(lines, key=lambda user: user['cell'][sortnames[c.sort]] , reverse=reverse)
+
+            lines = sorted(lines,
+                           key=lambda user: user['cell'][sortnames[c.sort]],
+                           reverse=reverse,
+                           cmp=unicode_compare)
             # end: sorting
 
             # reducing the page
@@ -499,14 +500,12 @@ class ManageController(BaseController):
             return json.dumps(res, indent=3)
 
         except PolicyException as pe:
-            log.error("[userview_flexi] Error during checking policies: %r" % pe)
-            log.error("[userview_flexi] %s" % traceback.format_exc())
+            log.exception("[userview_flexi] Error during checking policies: %r" % pe)
             Session.rollback()
             return sendError(response, unicode(pe), 1)
 
         except Exception as e:
-            log.error("[userview_flexi] failed: %r" % e)
-            log.error("[userview_flexi] %s" % traceback.format_exc())
+            log.exception("[userview_flexi] failed: %r" % e)
             Session.rollback()
             return sendError(response, e)
 
@@ -561,14 +560,12 @@ class ManageController(BaseController):
             return render('/manage/tokeninfo.mako')
 
         except PolicyException as pe:
-            log.error("[tokeninfo] Error during checking policies: %r" % pe)
-            log.error("[tokeninfo] %s" % traceback.format_exc())
+            log.exception("[tokeninfo] Error during checking policies: %r" % pe)
             Session.rollback()
             return sendError(response, unicode(pe), 1)
 
         except Exception as e:
-            log.error("[tokeninfo] failed! %r" % e)
-            log.error("[tokeninfo] %s" % traceback.format_exc())
+            log.exception("[tokeninfo] failed! %r" % e)
             Session.rollback()
             return sendError(response, e)
 
@@ -616,8 +613,7 @@ class ManageController(BaseController):
             return wsgi_app(request.environ, self.start_response)
 
         except Exception as e:
-            log.error("[help] Error loading helpfile: %r" % e)
-            log.error("[help] %s" % traceback.format_exc())
+            log.exception("[help] Error loading helpfile: %r" % e)
             Session.rollback()
             return sendError(response, e)
 
@@ -695,9 +691,8 @@ def _getTokenTypeConfig(section='config'):
                 t_html = remove_empty_lines(t_html)
 
             except CompileException as ex:
-                log.error("[_getTokenTypeConfig] compile error while processing %r.%r:" % (tok, section))
+                log.exception("[_getTokenTypeConfig] compile error while processing %r.%r:" % (tok, section))
                 log.error("[_getTokenTypeConfig] %r" % ex)
-                log.error("[_getTokenTypeConfig] %s" % traceback.format_exc())
                 raise Exception(ex)
 
             except Exception as e:
