@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -36,6 +36,8 @@ import re
 import webob
 import binascii
 
+from useridresolver.UserIdResolver import ResolverLoadConfigError
+
 from linotp.lib.selftest import isSelfTest
 from pylons import request, response, config, tmpl_context as c
 
@@ -86,13 +88,15 @@ from linotp.lib.policy import getPolicyDefinitions
 from linotp.lib.policy import create_policy_export_file
 from linotp.lib.policy import get_client_policy
 
+import linotp.lib.support
+
+
 from paste.fileapp import FileApp
 from cgi import escape
 from pylons.i18n.translation import _
 
 audit = config.get('audit')
 
-import traceback
 import logging
 log = logging.getLogger(__name__)
 
@@ -131,35 +135,32 @@ class SystemController(BaseController):
             c.audit['success'] = False
             c.audit['client'] = get_client()
 
-            ## check session might raise an abort()
+            # check session might raise an abort()
             check_session()
 
             # check authorization
             if action not in ["_add_dynamic_tokens", 'setupSecurityModule',
-                              'getSupportInfo']:
+                              'getSupportInfo', 'isSupportValid']:
                 checkPolicyPre('system', action)
 
-            ## default return for the __before__ and __after__
+            # default return for the __before__ and __after__
             return response
 
         except PolicyException as pex:
-            log.error("[__before__::%r] policy exception %r" % (action, pex))
-            log.error("[__before__] %s" % traceback.format_exc())
+            log.exception("[__before__::%r] policy exception %r" % (action, pex))
             Session.rollback()
             Session.close()
             return sendError(response, pex, context='before')
 
         except webob.exc.HTTPUnauthorized as acc:
-            ## the exception, when an abort() is called if forwarded
-            log.error("[__before__::%r] webob.exception %r" % (action, acc))
-            log.error("[__before__] %s" % traceback.format_exc())
+            # the exception, when an abort() is called if forwarded
+            log.exception("[__before__::%r] webob.exception %r" % (action, acc))
             Session.rollback()
             Session.close()
             raise acc
 
         except Exception as exx:
-            log.error("[__before__::%r] exception %r" % (action, exx))
-            log.error("[__before__] %s" % traceback.format_exc())
+            log.exception("[__before__::%r] exception %r" % (action, exx))
             Session.rollback()
             Session.close()
             return sendError(response, exx, context='before')
@@ -178,12 +179,11 @@ class SystemController(BaseController):
         try:
             c.audit['administrator'] = getUserFromRequest(request).get("login")
             audit.log(c.audit)
-            ## default return for the __before__ and __after__
+            # default return for the __before__ and __after__
             return response
 
         except Exception as exx:
-            log.error("[__after__] exception %r" % (exx))
-            log.error("[__after__] %s" % traceback.format_exc())
+            log.exception("[__after__] exception %r" % (exx))
             Session.rollback()
             Session.close()
             return sendError(response, exx, context='after')
@@ -234,7 +234,7 @@ class SystemController(BaseController):
                 "DefaultResetFailCount"]
 
 
-        ### config settings from here
+        # config settings from here
         try:
             param = getLowerParams(request.params)
             log.info("[setDefault] saving default configuration: %r" % param)
@@ -259,8 +259,7 @@ class SystemController(BaseController):
             return sendResult(response, res)
 
         except Exception as exx:
-            log.error('[setDefault] commit failed: %r' % exx)
-            log.error("[setDefault] %s" % traceback.format_exc())
+            log.exception('[setDefault] commit failed: %r' % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -320,7 +319,7 @@ class SystemController(BaseController):
                 c.audit['info'] = "%s=%s" % (key, val)
 
             else:
-                ## we gather all key value pairs in the conf dict
+                # we gather all key value pairs in the conf dict
                 conf = {}
                 for key in param:
                     if key == 'session':
@@ -344,8 +343,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[setConfig] error saving config: %r" % exx)
-            log.error("[setConfig] %s" % traceback.format_exc())
+            log.exception("[setConfig] error saving config: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -382,8 +380,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[delConfig] error deleting config: %r" % exx)
-            log.error("[delConfig] %s" % traceback.format_exc())
+            log.exception("[delConfig] error deleting config: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -418,7 +415,7 @@ class SystemController(BaseController):
             if 'session' in param:
                 del param['session']
 
-            ## if there is no parameter, we return them all
+            # if there is no parameter, we return them all
             if len(param) == 0:
                 conf = getLinotpConfig()
                 keys = conf.keys()
@@ -437,8 +434,8 @@ class SystemController(BaseController):
                         else:
                             res[Key] = conf.get(key)
 
-                ## as we return the decrypted values, we could do this in place
-                ## and display the value under the original key
+                # as we return the decrypted values, we could do this in place
+                # and display the value under the original key
                 for key in keys:
                     if key.startswith("enclinotp."):
                         Key = key[len("enclinotp."):]
@@ -460,8 +457,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[getConfig] error getting config: %r" % exx)
-            log.error("[getConfig] %s" % traceback.format_exc())
+            log.exception("[getConfig] error getting config: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -496,7 +492,7 @@ class SystemController(BaseController):
 
 
 
-        ### config settings from here
+        # config settings from here
         try:
             param = getLowerParams(request.params)
             log.debug("[getRealms] with params: %r" % param)
@@ -513,14 +509,12 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except PolicyException as pex:
-            log.error("[getRealms] policy exception: %r" % pex)
-            log.error("[getRealms] %s" % traceback.format_exc())
+            log.exception("[getRealms] policy exception: %r" % pex)
             Session.rollback()
             return sendError(response, pex)
 
         except Exception as exx:
-            log.error("[getRealms] error getting realms: %r" % exx)
-            log.error("[getRealms] %s" % traceback.format_exc())
+            log.exception("[getRealms] error getting realms: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -583,9 +577,14 @@ class SystemController(BaseController):
             Session.commit()
             return sendResult(response, res, 1)
 
+        except ResolverLoadConfigError as exx:
+            log.exception("Failed to load resolver definition %r \n %r"
+                      % (exx, param))
+            Session.rollback()
+            return sendError(response, exx)
+
         except Exception as exx:
-            log.error("[setResolver] error saving config: %r" % exx)
-            log.error("[setResolver] %s" % traceback.format_exc())
+            log.exception("[setResolver] error saving config: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -622,8 +621,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[getResolvers] error getting resolvers: %r" % exx)
-            log.error("[getResolvers] %s" % traceback.format_exc())
+            log.exception("[getResolvers] error getting resolvers: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -658,7 +656,7 @@ class SystemController(BaseController):
             log.info("[delResolver] deleting resolver: %r" % param)
 
             resolver = getParam(param, "resolver", required)
-            ### only delete a resolver, if it is not used by any realm
+            # only delete a resolver, if it is not used by any realm
             found = False
             fRealms = []
             realms = getRealms()
@@ -687,8 +685,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[delResolver] error deleting resolver: %r" % exx)
-            log.error("[delResolver] %s" % traceback.format_exc())
+            log.exception("[delResolver] error deleting resolver: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -735,8 +732,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[getResolver] error getting resolver: %r" % exx)
-            log.error("[getResolver] %s" % traceback.format_exc())
+            log.exception("[getResolver] error getting resolver: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -785,8 +781,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[setDefaultRealm] setting default realm failed: %r" % exx)
-            log.error("[setDefaultRealm] %s" % traceback.format_exc())
+            log.exception("[setDefaultRealm] setting default realm failed: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -825,8 +820,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[getDefaultRealm] return default realm failed: %r" % exx)
-            log.error("[getDefaultRealm] %s" % traceback.format_exc())
+            log.exception("[getDefaultRealm] return default realm failed: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -870,11 +864,12 @@ class SystemController(BaseController):
 
             realm_resolvers = []
             for resolver in resolvers.split(','):
-                ## check resolver returns the correct resolver description
+                # check resolver returns the correct resolver description
                 (res, realm_resolver) = checkResolverType(resolver)
                 if res == False:
                     raise Exception("unknown resolver %r  or invalid resolver "
-                                    "class specification " % resolver)
+                                    "class specification: %r "
+                                    % (resolver, realm_resolver))
                 realm_resolvers.append(realm_resolver)
 
             resolvers = ",".join(realm_resolvers)
@@ -887,8 +882,7 @@ class SystemController(BaseController):
 
         except Exception as exx:
             err = ("Failed to set realm with %r " % param)
-            log.error("[setRealm] %r %r" % (err, exx))
-            log.error("[setRealm] %s" % traceback.format_exc())
+            log.exception("[setRealm] %r %r" % (err, exx))
             Session.rollback()
             return sendError(response, exx)
 
@@ -924,15 +918,15 @@ class SystemController(BaseController):
             realm = getParam(param, "realm", required)
 
 
-            ## we test if before delete there has been a default
-            ## if yes - check after delete, if still one there
-            ##         and set the last available to default
+            # we test if before delete there has been a default
+            # if yes - check after delete, if still one there
+            #         and set the last available to default
             defRealm = getDefaultRealm()
             hadDefRealmBefore = False
             if defRealm != "":
                 hadDefRealmBefore = True
 
-            ## now test if realm is defined
+            # now test if realm is defined
             if isRealmDefined(realm) == True:
                 if realm.lower() == defRealm.lower():
                     setDefaultRealm("")
@@ -959,8 +953,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[delRealm] error deleting realm: %r" % exx)
-            log.error("[delRealm] %s" % traceback.format_exc())
+            log.exception("[delRealm] error deleting realm: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1014,9 +1007,10 @@ class SystemController(BaseController):
 
             # check that the name does not contain a .
             if not re.match('^[a-zA-Z0-9_]*$', name):
-                raise Exception (_("The name of the policy may only contain the characters a-zA-Z0-9_"))
+                raise Exception(_("The name of the policy may only contain "
+                                   "the characters a-zA-Z0-9_"))
             if not name:
-                raise Exception (_("The name of the policy must not be empty"))
+                raise Exception(_("The name of the policy must not be empty"))
 
             action = getParam(param, "action", required)
             scope = getParam(param, "scope", required)
@@ -1024,21 +1018,26 @@ class SystemController(BaseController):
             user = getParam(param, "user", optional)
             time = getParam(param, "time", optional)
             client = getParam(param, "client", optional)
-            active = getParam(param, "active", optional)
+            active = param.get("active", 'True')
 
-            p_param = { 'name': name,
-                      'action' : action,
-                      'scope' : scope,
-                      'realm' : realm,
-                      'user' : user,
-                      'time' : time,
+            p_param = {'name': name,
+                      'action': action,
+                      'scope': scope,
+                      'realm': realm,
+                      'user': user,
+                      'time': time,
                       'client': client,
-                      'active' : active}
+                      'active': active
+                      }
+
+            enforce = param.get('enforce', 'False')
+            if enforce.lower() == 'true':
+                enforce = True
+                p_param['enforce'] = enforce
 
             c.audit['action_detail'] = unicode(param)
 
             if len(name) > 0 and len(action) > 0:
-
                 log.debug("[setPolicy] saving policy %r" % p_param)
                 ret = setPolicy(p_param)
                 log.debug("[setPolicy] policy %s successfully saved." % name)
@@ -1061,8 +1060,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[setPolicy] error saving policy: %r" % exx)
-            log.error("[setPolicy] %s" % traceback.format_exc())
+            log.exception("[setPolicy] error saving policy: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1133,8 +1131,7 @@ class SystemController(BaseController):
             return json.dumps(res, indent=3)
 
         except Exception as exx:
-            log.error("[policies_flexi] error in policy flexi: %r" % exx)
-            log.error("[policies_flexi] %s" % traceback.format_exc())
+            log.exception("[policies_flexi] error in policy flexi: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1182,8 +1179,7 @@ class SystemController(BaseController):
             return sendResult(response, pol, 1)
 
         except Exception as exx:
-            log.error("[getPolicyDef] error getting policy definitions: %r" % exx)
-            log.error("[getPolicyDef] %s" % traceback.format_exc())
+            log.exception("[getPolicyDef] error getting policy definitions: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1214,7 +1210,7 @@ class SystemController(BaseController):
             tclass = tokenclasses.get(tok)
             tclass_object = newToken(tclass)
             if hasattr(tclass_object, 'getClassInfo'):
-                ## check if we have a policy in the definition
+                # check if we have a policy in the definition
                 try:
                     policy = tclass_object.getClassInfo('policy', ret=None)
                     if policy is not None and policy.has_key(scope):
@@ -1287,8 +1283,7 @@ class SystemController(BaseController):
             return sendResultMethod(response, res)
 
         except Exception as exx:
-            log.error("[importPolicy] failed! %r" % exx)
-            log.error("[importPolicy] %s" % traceback.format_exc())
+            log.exception("[importPolicy] failed! %r" % exx)
             Session.rollback()
             return sendErrorMethod(response, exx)
 
@@ -1369,8 +1364,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[checkPolicy] error checking policy: %r" % exx)
-            log.error("[checkPolicy] %s" % traceback.format_exc())
+            log.exception("[checkPolicy] error checking policy: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1391,6 +1385,8 @@ class SystemController(BaseController):
         arguments:
             * realm - (optional) will return all policies in the given realm
             * name  - (optional) will only return the policy with the given name
+            * action  (optional) will only return the policy with the given action
+            * user    (optional) will only return the policy for this user
             * scope - (optional) will only return the policies within the given scope
             * export - (optional) The filename needs to be specified as the third part of the URL like /system/getPolicy/policy.cfg. It
                     will then be exported to this file.
@@ -1411,12 +1407,18 @@ class SystemController(BaseController):
         log.debug("[getPolicy] getting policy: %r" % param)
         export = None
 
-        ### config settings from here
-
+        # config settings from here
+        action = None
+        user = None
         try:
             name = getParam(param, "name", optional)
             realm = getParam(param, "realm", optional)
             scope = getParam(param, "scope", optional)
+            if 'action' in param:
+                action = param.get('action') or None
+            if 'user' in param:
+                user = param.get('user') or None
+
             display_inactive = getParam(param, "display_inactive", optional)
             if display_inactive:
                 display_inactive = True
@@ -1429,10 +1431,30 @@ class SystemController(BaseController):
             pol = {}
             if name != None:
                 for nam in name.split(','):
-                    poli = getPolicy({'name':nam, 'realm':realm, 'scope': scope}, display_inactive=display_inactive)
+                    search_param = {'name':nam, 'realm':realm, 'scope': scope}
+                    if action:
+                        search_param['action'] = action
+                    poli = getPolicy(search_param, display_inactive=display_inactive)
                     pol.update(poli)
             else:
-                pol = getPolicy({'name':name, 'realm':realm, 'scope': scope}, display_inactive=display_inactive)
+                search_param = {'name':name, 'realm':realm, 'scope': scope}
+                if action:
+                    search_param['action'] = action
+                pol = getPolicy(search_param, display_inactive=display_inactive)
+
+            # due to bug in getPolicy we have to post check if user is in policy!
+            if user:
+                rpol = {}
+                for p_name, policy in pol.items():
+                    if policy['user'] == None:
+                        rpol[p_name] = policy
+                    else:
+                        users = policy['user'].split(',')
+                        for use in users:
+                            if use.strip() == user.strip() or use.strip() == '*':
+                                rpol[p_name] = policy
+                pol = rpol
+
 
             c.audit['success'] = True
             c.audit['info'] = "name = %s, realm = %s, scope = %s" \
@@ -1448,8 +1470,7 @@ class SystemController(BaseController):
                 return sendResult(response, pol, 1)
 
         except Exception as exx:
-            log.error("[getPolicy] error getting policy: %r" % exx)
-            log.error("[getPolicy] %s" % traceback.format_exc())
+            log.exception("[getPolicy] error getting policy: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1477,14 +1498,25 @@ class SystemController(BaseController):
 
         """
         res = {}
+        ret = {}
+        param = {}
         try:
-            param = getLowerParams(request.params)
+            param.update(request.params)
             log.info("[delPolicy] deleting policy: %r" % param)
 
-            name = getParam(param, "name", required)
+            # support the ignor of policy impact check
+            enforce = param.get("enforce", 'False')
+            if enforce.lower() == 'true':
+                enforce = True
+            else:
+                enforce = False
+
+            name_param = param["name"]
+            names = name_param.split(',')
+            for name in names:
+                log.debug("[delPolicy] trying to delete policy %s" % name)
+                ret.update(deletePolicy(name, enforce))
 
-            log.debug("[delPolicy] trying to delete policy %s" % name)
-            ret = deletePolicy(name)
             res["delPolicy"] = {"result": ret}
 
             c.audit['success'] = ret
@@ -1494,8 +1526,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[delPolicy] error deleting policy: %r" % exx)
-            log.error("[delPolicy] %s" % traceback.format_exc())
+            log.exception("[delPolicy] error deleting policy: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1520,7 +1551,7 @@ class SystemController(BaseController):
             glo = getGlobalObject()
             sep = glo.security_provider
 
-            ## for test purpose we switch to an errHSM
+            # for test purpose we switch to an errHSM
             if isSelfTest():
                 if params.get('__hsmexception__') == '__ON__':
                     hsm = c.hsm.get('obj')
@@ -1564,8 +1595,7 @@ class SystemController(BaseController):
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[setupSecurityModule] : setup failed: %r" % exx)
-            log.error("[setupSecurityModule] %s" % traceback.format_exc())
+            log.exception("[setupSecurityModule] : setup failed: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1581,48 +1611,15 @@ class SystemController(BaseController):
         or the support subscription info, which could be the old license
         """
         res = {}
-        params = {}
         try:
-            params.update(request.params)
-            log.debug("[getSupportInfo] parameters: %r" % params)
 
-            license_data = getFromConfig("license")
+            res = linotp.lib.support.getSupportLicenseInfo()
 
-            if license_data is not None and len(license_data) > 0:
-                support_string = binascii.unhexlify(license_data)
-                (support_info, sign, licStr) = parseSubscription(support_string)
-
-                res['description'] = support_info
-
-            else:
-
-                version = get_version_number()
-
-                support_info = {}
-                support_info['comment'] = (_('LinOTP Support Info'))
-                support_info['version'] = 'LinOTP %s' % version
-                support_info['issuer'] = ''
-                support_info['token-num'] = ''
-                support_info['licensee'] = ''
-                support_info['address'] = '<a href="http://www.lsexperts.de" target="_blank">http://www.lsexperts.de</a>'
-                support_info['contact-name'] = ''
-                support_info['contact-email'] = '<a href="mailto:linotp@lsexperts.de">linotp@lsexperts.de</a>'
-                support_info['contact-phone'] = '+49 6151 86086-115'
-                support_info['date'] = ''
-                support_info['expire'] = ''
-                support_info['subscription'] = (
-                    _('You are using the open source version with community '
-                    'support. For professional support, feel free to contact '
-                    'LSE by email or by phone.'))
-                res['description'] = support_info
-
-            c.audit['success'] = res
-            Session.commit()
+            c.audit['success'] = True
             return sendResult(response, res, 1)
 
         except Exception as exx:
-            log.error("[getSupportInfo] : failed to access support info: %r" % exx)
-            log.error("[getSupportInfo] %s" % traceback.format_exc())
+            log.exception("[getSupportInfo] : failed to access support info: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
@@ -1630,15 +1627,49 @@ class SystemController(BaseController):
             Session.close()
             log.error("[getSupportInfo] done")
 
+    def isSupportValid(self):
+        """
+        verifies the support license status
+
+        if ok
+            status and value in response are both true
+        else
+            value is false and the detail is returned as detail in the response
+        """
+        res = {}
+        message = None
+        try:
+            license_txt = getFromConfig('license', '')
+            try:
+                licString = binascii.unhexlify(license_txt)
+            except TypeError:
+                licString = license_txt
+
+            (res, msg) = linotp.lib.support.isSupportLicenseValid(licString)
+            if res == False:
+                message = {'reason':msg}
+
+            c.audit['success'] = res
+            return sendResult(response, res, 1, opt=message)
+
+        except Exception as exx:
+            log.exception("[isSupportValid] failed verify support info: %r" % exx)
+            Session.rollback()
+            return sendError(response, exx)
+
+        finally:
+            Session.close()
+            log.debug("[isSupportValid] done")
+
     def setSupport(self):
         """
         hook to load a support subscription file
 
         receives the data with a form post file upload
+        and installes it after license verification
         """
         res = False
-        params = {}
-        support_description = ""
+        message = None
 
         try:
             licField = request.POST['license']
@@ -1653,66 +1684,25 @@ class SystemController(BaseController):
                 support_description = licField.encode('utf-8')
             log.debug("[setSupport] license %s", support_description)
 
-            storeConfig("license", binascii.hexlify(support_description))
 
-            res = True
+            res, msg = linotp.lib.support.setSupportLicense(support_description)
+            if res == False:
+                message = {'reason' : msg}
+
             c.audit['success'] = res
 
             Session.commit()
-            return sendResult(response, res, 1)
+            return sendResult(response, res, 1, opt=message)
 
         except Exception as exx:
-            log.error("[setSupport] : failed to acces support info: %r" % exx)
-            log.error("[setSupport] %s" % traceback.format_exc())
+            log.exception("[setSupport] failed to set support license: %r" % exx)
             Session.rollback()
             return sendError(response, exx)
 
         finally:
             Session.close()
             log.error("[setSupport] done")
-        return valid;
-
-### helper function
-
-def parseSubscription(licString):
-    """
-    parse the support subscription license
-
-    :param licString: the support license as multiline string
-    :return: tuple of license dict, extracted signature and the
-                      license as string, which the signature could be checked
-                      against
-    """
-    licInfo = {}
-    signature = ""
-    licStr = ""
 
 
-    read_license = 0
-    read_signature = 0
-
-    for l in licString.split('\n'):
-        if (l.startswith("-----BEGIN LICENSE-----")):
-            read_license = 1
-        elif (l.startswith("-----END LICENSE-----")):
-            read_license = 0
-        elif (l.startswith("-----BEGIN LICENSE SIGNATURE-----")):
-            read_signature = 1
-        elif (l.startswith("-----END LICENSE SIGNATURE-----")):
-            read_signature = 0
-        else:
-            if 1 == read_license:
-                licStr = licStr + l + "\n"
-                try:
-                    (key, val) = l.split("=", 2)
-                    licInfo[key] = val
-                except:
-                    log.debug("->parseLicense - %s", l)
-            if 1 == read_signature:
-                signature += l.rstrip()
-
-    log.debug("->p - :::%s:::", licStr)
-
-    return (licInfo, signature, licStr)
 #eof###########################################################################
 
