@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -106,8 +106,7 @@ class ValidateController(BaseController):
             return response
 
         except Exception as exx:
-            log.error("[__before__::%r] exception %r" % (action, exx))
-            log.error("[__before__] %s" % traceback.format_exc())
+            log.exception("[__before__::%r] exception %r" % (action, exx))
             Session.rollback()
             Session.close()
             return sendError(response, exx, context='before')
@@ -252,11 +251,13 @@ class ValidateController(BaseController):
 
             Session.commit()
 
-            qr = getParam(param, 'qr', optional)
-            if qr is not None and opt is not None and opt.has_key('message'):
+            qr = param.get('qr', None)
+            if qr and opt and 'message' in opt:
                 try:
                     dataobj = opt.get('message')
                     param['alt'] = "%s" % opt
+                    if 'transactionid' in opt:
+                        param['transactionid'] = opt['transactionid']
                     return sendQRImageResult(response, dataobj, param)
                 except Exception as exc:
                     log.warning("failed to send QRImage: %r " % exc)
@@ -265,14 +266,11 @@ class ValidateController(BaseController):
                 return sendResult(response, ok, 0, opt=opt)
 
         except Exception as exx:
-            log.error("[check] validate/check failed: %r" % exx)
-            log.error("[check] %s" % traceback.format_exc())
+            log.exception("[check] validate/check failed: %r" % exx)
             # If an internal error occurs or the SMS gateway did not send the SMS, we write this to the detail info.
             c.audit['info'] = unicode(exx)
-
             Session.rollback()
-            return sendError(response, u"validate/check failed: %s"
-                             % unicode(exx), 0)
+            return sendResult(response, False, 0)
 
         finally:
             Session.close()
@@ -329,12 +327,10 @@ class ValidateController(BaseController):
             return sendResult(response, ok, 0, opt=opt)
 
         except Exception as exx:
-            log.error("[check_yubikey] validate/check_yubikey failed: %r" % exx)
-            log.error("[check_yubikey] %s" % traceback.format_exc())
+            log.exception("[check_yubikey] validate/check_yubikey failed: %r" % exx)
             c.audit['info'] = unicode(exx)
             Session.rollback()
-            return sendError(response, u"validate/check_yubikey failed: %s"
-                             % unicode(exx), 0)
+            return sendResult(response, False, 0)
 
         finally:
             Session.close()
@@ -368,17 +364,14 @@ class ValidateController(BaseController):
 
         except webob.exc.HTTPUnauthorized as acc:
             ## the exception, when an abort() is called if forwarded
-            log.error("[__before__::%r] webob.exception %r" % acc)
-            log.error("[__before__] %s" % traceback.format_exc())
+            log.exception("[__before__::%r] webob.exception %r" % acc)
             Session.rollback()
             raise acc
 
         except Exception as exx:
-            log.error("[check_url] validate/check_url failed: %r" % exx)
-            log.error("[check_url] %s" % traceback.format_exc())
+            log.exception("[check_url] validate/check_url failed: %r" % exx)
             Session.rollback()
-            return sendError(response, u"validate/check_url failed: %s"
-                             % unicode(exx), 0)
+            return sendResult(response, False, 0)
 
         finally:
             Session.close()
@@ -439,11 +432,9 @@ class ValidateController(BaseController):
             return sendResult(response, { 'auth': ok, 'attributes' : attributes } , 0, opt)
 
         except Exception as exx:
-            log.error("[samlcheck] validate/check failed: %r" % exx)
-            log.error("[samlcheck] %s" % traceback.format_exc())
+            log.exception("[samlcheck] validate/check failed: %r" % exx)
             Session.rollback()
-            return sendError(response, "validate/samlcheck failed: %s"
-                             % unicode(exx), 0)
+            return sendResult(response, False, 0)
 
         finally:
             Session.close()
@@ -508,11 +499,13 @@ class ValidateController(BaseController):
             #c.audit['info'] += "%s=%s, " % (k, value)
             Session.commit()
 
-            qr = getParam(param, 'qr', optional)
-            if qr is not None and opt is not None and opt.has_key('message'):
+            qr = param.get('qr', None)
+            if qr and opt and 'message' in opt:
                 try:
                     dataobj = opt.get('message')
                     param['alt'] = "%s" % opt
+                    if 'transactionid' in opt:
+                        param['transactionid'] = opt['transactionid']
                     return sendQRImageResult(response, dataobj, param)
                 except Exception as exc:
                     log.warning("failed to send QRImage: %r " % exc)
@@ -521,12 +514,10 @@ class ValidateController(BaseController):
                 return sendResult(response, value, 1, opt=opt)
 
         except Exception as exx:
-            log.error("[check_t] validate/check_t failed: %r" % exx)
-            log.error("[check_t] %s" % traceback.format_exc())
+            log.exception("[check_t] validate/check_t failed: %r" % exx)
             c.audit['info'] = unicode(exx)
             Session.rollback()
-            return sendError(response, "validate/check_t failed: %s"
-                             % unicode(exx), 0)
+            return sendResult(response, False, 0)
 
         finally:
             Session.close()
@@ -601,11 +592,13 @@ class ValidateController(BaseController):
             c.audit['success'] = ok
             Session.commit()
 
-            qr = getParam(param, 'qr', optional)
-            if qr is not None and opt is not None and opt.has_key('message'):
+            qr = param.get('qr', None)
+            if qr and opt and 'message' in opt:
                 try:
                     dataobj = opt.get('message')
                     param['alt'] = "%s" % opt
+                    if 'transactionid' in opt:
+                        param['transactionid'] = opt['transactionid']
                     return sendQRImageResult(response, dataobj, param)
                 except Exception as exc:
                     log.warning("failed to send QRImage: %r " % exc)
@@ -614,12 +607,10 @@ class ValidateController(BaseController):
                 return sendResult(response, ok, 0, opt=opt)
 
         except Exception as exx:
-            log.error("[check_s] validate/check_s failed: %r" % exx)
-            log.error("[check_s] %s" % traceback.format_exc())
+            log.exception("[check_s] validate/check_s failed: %r" % exx)
             c.audit['info'] = unicode(exx)
             Session.rollback()
-            return sendError(response, "validate/check_s failed: %s"
-                             % unicode(exx), 0)
+            return sendResult(response, False, 0)
 
         finally:
             Session.close()
@@ -685,21 +676,19 @@ class ValidateController(BaseController):
             return " ".join(res).strip()
 
         except Exception as exx:
-            log.error("[simplecheck] failed: %r" % exx)
-            log.error("[simplecheck] %s" % traceback.format_exc())
+            log.exception("[simplecheck] failed: %r" % exx)
             Session.rollback()
-            return u":-/"
+            return u":-("
 
         finally:
             Session.close()
             log.debug('[simplecheck] done')
 
-
     def ok(self):
-        return sendResult(response, "TRUE", 0)
+        return sendResult(response, True, 0)
 
     def fail(self):
-        return sendResult(response, "FALSE", 0)
+        return sendResult(response, False, 0)
 
     def smspin(self):
         '''
@@ -756,13 +745,11 @@ class ValidateController(BaseController):
             return sendResult(response, ret, opt)
 
         except Exception as exx:
-            log.error("[smspin] validate/smspin failed: %r" % exx)
-            log.error("[smspin] %s" % traceback.format_exc())
+            log.exception("[smspin] validate/smspin failed: %r" % exx)
             # If an internal error occurs or the SMS gateway did not send the SMS, we write this to the detail info.
             c.audit['info'] = unicode(exx)
             Session.rollback()
-            return sendError(response, "validate/smspin failed: %s"
-                             % unicode(exx), 0)
+            return sendResult(response, False, 0)
 
         finally:
             Session.close()
