@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -38,9 +38,6 @@ uses a public/private key for signing the log entries
 import datetime
 from sqlalchemy import schema, types, orm, and_, or_, asc, desc
 
-## TODO: the wildcard import is bad!!
-from migrate import *
-
 from M2Crypto import EVP, RSA
 from binascii import hexlify
 from binascii import unhexlify
@@ -49,17 +46,21 @@ from linotp.lib.audit.base import AuditBase
 from pylons import config
 
 import logging.config
-import traceback
+
 
 import linotp
 
 # Create the logging object from the linotp.ini config file
 ini_file = config.get("__file__")
-logging.config.fileConfig(ini_file, disable_existing_loggers=False)
+if ini_file is not None:
+    # When importing the module with Sphinx to generate documentation
+    # 'ini_file' is None. In other cases this should not be the case.
+    logging.config.fileConfig(ini_file, disable_existing_loggers=False)
 log = logging.getLogger(__name__)
 
 metadata = schema.MetaData()
 
+
 def now():
     u_now = u"%s" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
     return u_now
@@ -94,6 +95,7 @@ AUDIT_ENCODE = ["action", "serial", "success", "user", "realm", "tokentype",
                 "administrator", "action_detail", "info", "linotp_server",
                 "client", "log_level"]
 
+
 class AuditTable(object):
 
     def __init__(self, serial=u"", action=u"", success=u"False",
@@ -183,10 +185,13 @@ class AuditTable(object):
             field_len = self._get_field_len(name)
             encoded_value = linotp.lib.crypt.uencode(value)
             if field_len != -1 and len(encoded_value) > field_len:
-                log.warning("truncating audit data: [audit.%s] %s" % (name, value))
-                trunc_as_err = config.get("linotpAudit.error_on_truncation", False) or False
+                log.warning("truncating audit data: [audit.%s] %s"
+                            % (name, value))
+                trunc_as_err = config.get("linotpAudit.error_on_truncation",
+                                          False) or False
                 if trunc_as_err != False:
-                    raise Exception("truncating audit data: [audit.%s] %s" % (name, value))
+                    raise Exception("truncating audit data: [audit.%s] %s"
+                                    % (name, value))
 
                 ## during the encoding the value might expand -
                 ## so we take this additional length into account
@@ -220,8 +225,43 @@ class AuditTable(object):
 
 orm.mapper(AuditTable, audit_table)
 
-########################################################################################
 
+# replace sqlalchemy-migrate by the ability to ad a column
+def add_column(engine, table, column):
+    """
+    small helper to add a column by calling a native 'ALTER TABLE' to
+    replace the need for sqlalchemy-migrate
+
+    from:
+    http://stackoverflow.com/questions/7300948/add-column-to-sqlalchemy-table
+
+    :param engine: the running sqlalchemy
+    :param table: in which table should this column be added
+    :param column: the sqlalchemy definition of a column
+
+    :return: boolean of success or not
+    """
+
+    result = False
+
+    table_name = table.description
+    column_name = column.compile(dialect=engine.dialect)
+    column_type = column.type.compile(engine.dialect)
+
+    try:
+        engine.execute('ALTER TABLE %s ADD COLUMN %s %s'
+                                % (table_name, column_name, column_type))
+        result = True
+
+    except Exception as exx:
+        # Obviously we already migrated the database.
+        log.info("[__init__] Error during database migration: %r" % exx)
+        result = False
+
+    return result
+
+
+###############################################################################
 class Audit(AuditBase):
     """
     Audit Implementation to the generic audit interface
@@ -265,14 +305,11 @@ class Audit(AuditBase):
         self.VerifyEVP.reset_context(md='sha256')
         self.VerifyEVP.assign_rsa(self.PublicKey)
 
-        try:
-            # create the column "client"
-            column = schema.Column("client", types.Unicode(80))
-            column.create(audit_table)
-        except Exception as exx:
-            # Obviously we already migrated the database.
-            log.info("[__init__] Error during database migration: %r" % exx)
+        # create the column "client"
+        column = schema.Column("client", types.Unicode(80))
+        add_column(self.engine, audit_table, column)
 
+        return
 
     def _attr_to_dict(self, audit_line):
 
@@ -316,7 +353,6 @@ class Audit(AuditBase):
         log.debug("[_sign] signature : %s" % hexlify(signature))
         return hexlify(signature)
 
-
     def _verify(self, auditline, signature):
         '''
         Verify the signature of the audit line
@@ -360,8 +396,7 @@ class Audit(AuditBase):
             log.debug("[log] writing log done!")
 
         except Exception as  exx:
-            log.error("[log] error writing log message: %r" % exx)
-            log.error("[log] %s" % traceback.format_exc())
+            log.exception("[log] error writing log message: %r" % exx)
             self.session.rollback()
             raise exx
 
@@ -399,7 +434,6 @@ class Audit(AuditBase):
         self.session.merge(at)
         self.session.flush()
 
-
     def initialize_log(self, param):
         '''
         This method initialized the log state.
@@ -415,13 +449,11 @@ class Audit(AuditBase):
         '''
         pass
 
-
     def _buildCondition(self, param, AND):
         '''
         create the sqlalchemy condition from the params
         '''
-        condition = None
-
+        conditions = []
         boolCheck = and_
         if not AND:
             boolCheck = or_
@@ -432,47 +464,38 @@ class Audit(AuditBase):
         for k, v in param.items():
             if "" != v:
                 if "serial" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.serial.like(v))
+                    conditions.append(AuditTable.serial.like(v))
                 elif "user" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.user.like(v))
+                    conditions.append(AuditTable.user.like(v))
                 elif "realm" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.realm.like(v))
+                    conditions.append(AuditTable.realm.like(v))
                 elif "action" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.action.like(v))
+                    conditions.append(AuditTable.action.like(v))
                 elif "action_detail" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.action_detail.like(v))
+                    conditions.append(AuditTable.action_detail.like(v))
                 elif "date" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.timestamp.like(v))
+                    conditions.append(AuditTable.timestamp.like(v))
                 elif "number" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.id.like(v))
+                    conditions.append(AuditTable.id.like(v))
                 elif "success" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.success.like(v))
+                    conditions.append(AuditTable.success.like(v))
                 elif "tokentype" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.tokentype.like(v))
+                    conditions.append(AuditTable.tokentype.like(v))
                 elif "administrator" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.administrator.like(v))
+                    conditions.append(AuditTable.administrator.like(v))
                 elif "info" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.info.like(v))
+                    conditions.append(AuditTable.info.like(v))
                 elif "linotp_server" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.linotp_server.like(v))
+                    conditions.append(AuditTable.linotp_server.like(v))
                 elif "client" == k:
-                    condition = boolCheck(condition,
-                                      AuditTable.client.like(v))
+                    conditions.append(AuditTable.client.like(v))
 
-        log.debug("[_buildCondition] return %s" % condition)
-        return condition
+        all_conditions = None
+        if conditions:
+            all_conditions = boolCheck(*conditions)
+
+        log.debug("[_buildCondition] return %s" % all_conditions)
+        return all_conditions
 
     def row2dict(self, audit_line):
         """
@@ -502,7 +525,6 @@ class Audit(AuditBase):
         else:
             line['sig_check'] = "FAIL"
 
-
         return line
 
     def searchQuery(self, param, AND=True, display_error=True, rp_dict=None):
@@ -608,8 +630,6 @@ class Audit(AuditBase):
         result = self.session.execute(audit_q.statement)
         return result
 
-
-
     def getTotal(self, param, AND=True, display_error=True):
         '''
         This method returns the total number of audit entries in
@@ -625,13 +645,15 @@ class Audit(AuditBase):
         log.debug("[getTotal] count=%s " % str(c))
         return c
 
+
 def getAsString(data):
     '''
     We need to distinguish, if this is an entry after the adding the
     client entry or before. Otherwise the old signatures will break!
     '''
 
-    s = "number=%s, date=%s, action=%s, %s, serial=%s, %s, user=%s, %s, admin=%s, %s, %s, server=%s, %s, %s" % (
+    s = ("number=%s, date=%s, action=%s, %s, serial=%s, %s, user=%s, %s,"
+         " admin=%s, %s, %s, server=%s, %s, %s") % (
                 str(data.get('id')), str(data.get('timestamp')),
                 data.get('action'), str(data.get('success')),
                 data.get('serial'), data.get('tokentype'),
