@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -194,6 +194,12 @@ class Pkcs11SecurityModule(SecurityModule):
         library = config.get("library")
         self.slotid = int(config.get("slotid", 0))
 
+        # Accept invalid padding?
+        config_entry = config.get('pkcs11.accept_invalid_padding', 'False')
+        self.accept_invalid_padding = False
+        if config_entry and config_entry.lower() == 'true':
+            self.accept_invalid_padding = True
+
         self.handles = { CONFIG_KEY: config.get("configHandle", None),
                          TOKEN_KEY: config.get("tokenHandle", None),
                          VALUE_KEY: config.get("valueHandle", None),
@@ -250,41 +256,66 @@ class Pkcs11SecurityModule(SecurityModule):
 
         return
 
-    @classmethod
-    def pad(cls, s, block=16):
-        '''
+    def pad(self, unpadded_str, block=16):
+        """
         PKCS7 padding pads the missing bytes with the value of the number of the bytes.
         If 4 bytes are missing, this missing bytes are filled with \x04
-        '''
-        r = s
-        l_s = len(r)
+
+        :param unpadded_str: The string to pad
+        :type unpadded_str: str
+        :param block: Block size
+        :type block: int
+        :returns: padded string
+        :rtype: str
+        """
+        l_s = len(unpadded_str)
         missing_num = block - l_s % block
-        ## TODO: FIX: padding has to be elaborated
-        ##            with backward compatibility in mind
-        if missing_num == 16:
-            missing_num = 0
         missing_byte = chr(missing_num)
+        padding = missing_byte * missing_num
+        return unpadded_str + padding
 
-        r += missing_byte * missing_num
-        return r
+    def unpad(self, padded_str, block=16):
+        """
+        This removes and checks the PKCS #7 padding.
 
-    @classmethod
-    def unpad(cls, s, block=16):
-        '''
-        This removes the PKCS7 padding.
-        Check the last byte. If it is 1-15 we assume a padding and also
-        check the n last bytes
-        '''
-        if not s:
-            return s
-
-        r = s
-        last_byte = s[-1]
+        :param padded_str: The string to unpad
+        :type padded_str: str
+        :param block: Block size
+        :type block: int
+        :raises ValueError: If padded_str is not correctly padded a ValueError
+            can be raised.
+            This depends on the 'pkcs11.accept_invalid_padding' LinOTP config
+            option. If set to False (default) ValueError is raised.  The reason
+            why the data is sometimes incorrectly padded is because the pad()
+            method delivered with LinOTP version < 2.7.1 didn't pad correctly
+            when the data-length was a multiple of the block-length.
+            Beware that in some cases (statistically about 0.4% of data-chunks
+            whose length is a multiple of the block length) the incorrect
+            padding can not be detected and incomplete data is returned.  One
+            example for this last case is when the data ends with the byte
+            0x01. This is recognized as legitimate padding and is removed
+            before returning the data, thus removing a legitimate byte from the
+            data and making it unusable.
+            If you didn't upgrade from a LinOTP version before 2.7.1 (or don't
+            use a PKCS#11 HSM) you will not be affected by this in any way.
+            ValueError will of course also be raised if you data became corrupt
+            for some other reason (e.g. disk failure) and can not be unpadded.
+            In this case you should NOT set 'pkcs11.accept_invalid_padding' to
+            True because your data will be unusable anyway.
+        :returns: unpadded string or sometimes padded string when
+            'pkcs11.accept_invalid_padding' is set to True. See above.
+        :rtype: str
+        """
+        last_byte = padded_str[-1]
         count = ord(last_byte)
-        if count in range(1, 16):
-            if r[-ord(last_byte):] == last_byte * ord(last_byte):
-                r = r[:-ord(last_byte)]
-        return r
+        if 0 < count <= block and padded_str[-count:] == last_byte * count:
+            unpadded_str = padded_str[:-count]
+            return unpadded_str
+        elif self.accept_invalid_padding:
+            log.warning("[unpad] Input 'padded_str' is not properly padded")
+            return padded_str
+        else:
+            raise ValueError("Input 'padded_str' is not properly padded")
 
     def initpkcs11(self):
         '''
