@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -47,8 +47,6 @@ import time
 import hashlib
 import datetime
 
-import traceback
-
 
 from linotp.lib.error import TokenAdminError
 from linotp.lib.error import ParameterError
@@ -78,10 +76,8 @@ from linotp.model       import OcraChallenge
 
 from linotp.model.meta  import Session
 from linotp.lib.reply   import create_img
-from linotp.lib.apps    import create_google_authenticator_url
-from linotp.lib.apps    import create_oathtoken_url
 
-from linotp.lib.validate import check_pin
+from linotp.lib.validate import check_pin, is_same_transaction
 from linotp.lib.validate import check_otp
 from linotp.lib.validate import split_pin_otp
 
@@ -91,9 +87,9 @@ from sqlalchemy         import asc, desc
 from pylons.i18n.translation import _
 
 
-
 # needed for ocra token
 import urllib
+import linotp
 
 try:
     import json
@@ -106,13 +102,14 @@ required = False
 
 log = logging.getLogger(__name__)
 
+
 class TokenClass(object):
 
     def __init__(self, token):
         self.type = ''
         self.token = token
-        ## the info is a generic container, to store token specific processing info
-        ## which could be retrieved in the controllers
+        # the info is a generic container, to store token specific
+        # processing info which could be retrieved in the controllers
         self.info = {}
         self.hKeyRequired = False
         self.mode = ['authenticate', 'challenge']
@@ -122,6 +119,43 @@ class TokenClass(object):
         self.type = typ
         self.token.setType(typ)
 
+    def is_auth_only_token(self, user):
+        """
+        check if token is in the authenticate only mode
+        this is required to optimize the number of requests
+
+        :param user: the user / realm where the token policy is applied
+        :return: boolean
+        """
+        if len(self.mode) == 1 and 'authenticate' in self.mode:
+            return True
+
+        if len(self.mode) == 1 and 'challenge' in self.mode:
+            return False
+
+        import linotp.lib.policy
+        support_challenge_response = \
+            linotp.lib.policy.get_auth_challenge_response(user, self.type)
+
+        return not support_challenge_response
+
+    def is_challenge_and_auth_token(self, user):
+        """
+        check if token supports both authentication methods:
+          authenticate an challenge responser
+
+        :param user: the user / realm where the token policy is applied
+        :return: boolean
+        """
+
+        if 'authenticate' in self.mode and 'challenge' in self.mode:
+            import linotp.lib.policy
+            support_challenge_response = \
+                linotp.lib.policy.get_auth_challenge_response(user, self.type)
+            return support_challenge_response
+        else:
+            return False
+
     @classmethod
     def getClassType(cls):
         return None
@@ -130,6 +164,19 @@ class TokenClass(object):
     def getClassPrefix(cls):
         return "UNK"
 
+    def getRealms(self):
+
+        if hasattr(self, 'realms'):
+            return self.realms
+
+        realms = []
+        tokenrealms = self.token.getRealms()
+        for realm in tokenrealms:
+            realms.append(realm.name)
+
+        self.realms = realms
+        return self.realms
+
     def getType(self):
         return self.token.getType()
 
@@ -221,9 +268,17 @@ class TokenClass(object):
 
         (res, pin, otpval) = split_pin_otp(self, passw, user, options=options)
         if res != -1:
-            pin_match = check_pin(self, pin, user=user, options=options)
-            if pin_match is True:
+            pin_policies = linotp.lib.policy.get_pin_policies(user)
+            if 1 in pin_policies:
                 otp_counter = check_otp(self, otpval, options=options)
+                if otp_counter >= 0:
+                    pin_match = check_pin(self, pin, user=user, options=options)
+                    if not pin_match:
+                        otp_counter = -1
+            else:
+                pin_match = check_pin(self, pin, user=user, options=options)
+                if pin_match is True:
+                    otp_counter = check_otp(self, otpval, options=options)
 
         return (pin_match, otp_counter, reply)
 
@@ -420,7 +475,7 @@ class TokenClass(object):
         ## check if the transactionid is in the list of challenges
         if transid is not None:
             for challenge in challenges:
-                if challenge.getTransactionId() == transid:
+                if is_same_transaction(challenge, transid):
                     matching = challenge
                     break
             if matching is not None:
@@ -501,8 +556,6 @@ class TokenClass(object):
         Session.commit()
         return
 
-
-
     def update(self, param, reset_failcount=True):
 
         tdesc = getParam(param, "description", optional)
@@ -554,10 +607,15 @@ class TokenClass(object):
                 storeHashed = False
             self.token.setPin(pin, storeHashed)
 
-        otplen = getParam(param, 'otplen', optional)
-        if otplen is not None:
+        otplen = param.get('otplen', None)
+        if otplen:
             self.setOtpLen(otplen)
 
+        self.resetTokenInfo()
+
+        return
+
+    def resetTokenInfo(self):
         return
 
     def _genOtpKey_(self, otpkeylen=None):
@@ -599,6 +657,12 @@ class TokenClass(object):
     def getUser(self):
         uidResolver = self.token.LinOtpIdResolver or ''
         uidResolverClass = self.token.LinOtpIdResClass or ''
+
+        # we adjust the token-resolver-class-info to match
+        # to the available un-ee resolvers, which makes the live
+        # alot easier
+        if 'useridresolveree.' in uidResolverClass:
+            uidResolverClass = uidResolverClass.replace('useridresolveree.', 'useridresolver.')
         uuserid = self.token.LinOtpUserid or ''
         return (uuserid, uidResolver, uidResolverClass)
 
@@ -706,13 +770,12 @@ class TokenClass(object):
     def incOtpFailCounter(self):
         log.debug('incOtpFailCounter')
 
-        if self.token.LinOtpFailCount < self.token.LinOtpMaxFail:
-            self.token.LinOtpFailCount = self.token.LinOtpFailCount + 1
+        self.token.LinOtpFailCount = self.token.LinOtpFailCount + 1
 
         try:
             self.token.storeToken()
         except:
-            log.error('update failed')
+            log.exception('update failed')
             raise TokenAdminError("Token Fail Counter update failed", id=1106)
 
         return self.token.LinOtpFailCount
@@ -767,7 +830,7 @@ class TokenClass(object):
             try:
                 info = json.loads(tokeninfo)
             except Exception as e:
-                log.error('getTokenInfo %r' % (e))
+                log.exception('getTokenInfo %r' % (e))
 
         return info
 
@@ -800,6 +863,12 @@ class TokenClass(object):
             ret = info.get(key)
         return ret
 
+    def removeFromTokenInfo(self, key):
+        info = self.getTokenInfo()
+        if key in info:
+            del info[key]
+            self.setTokenInfo(info)
+
     # FIXME: we could store the
     #   count_auth_success_max
     #   count_auth_success
@@ -962,7 +1031,7 @@ class TokenClass(object):
 
         return True
 
-    def incOtpCounter(self, counter, reset=True):
+    def incOtpCounter(self, counter=None, reset=True):
         '''
         method
             incOtpCounter(aToken, counter)
@@ -983,6 +1052,9 @@ class TokenClass(object):
 
         resetCounter = False
 
+        if counter is None:
+            counter = self.token.LinOtpCount
+
         #CKO, FIX: in case of SMS Token, we always get a counter==0 ???
         #if 0 == counter:
         #    counter = self.token.LinOtpCount
@@ -1004,14 +1076,14 @@ class TokenClass(object):
             self.token.storeToken()
 
         except Exception as ex :
-            log.error("Token Counter update failed: %r" % (ex))
+            log.exception("Token Counter update failed: %r" % (ex))
             raise TokenAdminError("Token Counter update failed: %r" % (ex), id=1106)
 
         log.debug("[incOtpCounter] now got counter %s, %s" % (self.token.LinOtpCount, counter))
         return self.token.LinOtpCount
 
 
-    def check_otp_exist(self, otp, window=None):
+    def check_otp_exist(self, otp, window=None, user=None, autoassign=False):
         '''
         checks if the given OTP value is/are values of this very token.
         This is used to autoassign and to determine the serial number of
@@ -1021,8 +1093,6 @@ class TokenClass(object):
 
 
     def splitPinPass(self, passw):
-
-        res = 0
         try:
             otplen = int(self.token.LinOtpOtpLen)
         except ValueError:
@@ -1035,9 +1105,7 @@ class TokenClass(object):
             pin = passw[otplen:]
             otpval = passw[0:otplen]
 
-        #log.error("Pin: %s, otpval:%s",pin,otpval)
-
-        return (res, pin, otpval)
+        return pin, otpval
 
 
     def checkPin(self, pin, options=None):
@@ -1119,33 +1187,11 @@ class TokenClass(object):
 
         if otpkey != None:
             response_detail["otpkey"] = {
+                  "order"      : '1',
                   "description": _("OTP seed"),
                   "value"      :  "seed://%s" % otpkey,
                   "img"        :  create_img(otpkey, width=200),
                      }
-            if user is not None:
-                try:
-
-                    goo_url = create_google_authenticator_url(user.login,
-                                                  user.realm, otpkey,
-                                                  tok_type.lower(),
-                                                  serial=self.getSerial())
-                    response_detail["googleurl"] = {
-                          "description": _("URL for google Authenticator"),
-                          "value" :     goo_url,
-                          "img"   :     create_img(goo_url, width=250)
-                          }
-
-                    oath_url = create_oathtoken_url(user.login, user.realm,
-                                                    otpkey, tok_type,
-                                                    serial=self.getSerial())
-                    response_detail["oathurl"] = {
-                          "description" : _("URL for OATH token"),
-                           "value" : oath_url,
-                           "img"   : create_img(oath_url, width=250)
-                           }
-                except Exception as ex:
-                    log.info('failed to set oath or google url: %r' % ex)
 
         return response_detail
 
@@ -1168,7 +1214,7 @@ class TokenClass(object):
 
 
 class OcraTokenClass(TokenClass):
-    '''
+    """
     OcraTokenClass  implement an ocra compliant token
 
     used from Config
@@ -1178,49 +1224,46 @@ class OcraTokenClass(TokenClass):
         QrOcraDefaultSuite        - if none :'OCRA-1:HOTP-SHA256-8:C-QA64'
 
 
-    algorithm Ocra Token Rollout: tow phases of rollout
+    algorithm Ocra Token Rollout: two phases of rollout::
 
-    1. https://linotpserver/admin/init?
-        type=ocra&
-        genkey=1&
-        sharedsecret=1&
-        user=BENUTZERNAME&
-        session=SESSIONKEY
+        1. https://linotpserver/admin/init?
+            type=ocra&
+            genkey=1&
+            sharedsecret=1&
+            user=BENUTZERNAME&
+            session=SESSIONKEY
 
-        =>> "serial" : SERIENNUMMER, "sharedsecret" : DATAOBJECT, "app_import" : IMPORTURL
-        - genSharedSecret - vom HSM oder urandom ?
-        - app_import : + linotp://
-                       + ocrasuite ->> default aus dem config: (DefaultOcraSuite)
-                       + sharedsecret (Länge wie ???)
-                       + seriennummer
-        - seriennummer: uuid
-        - token wird angelegt ist aber nicht aktiv!!! (counter == 0)
+            =>> "serial" : SERIENNUMMER, "sharedsecret" : DATAOBJECT, "app_import" : IMPORTURL
+            - genSharedSecret - vom HSM oder urandom ?
+            - app_import : + linotp://
+                        + ocrasuite ->> default aus dem config: (DefaultOcraSuite)
+                        + sharedsecret (Länge wie ???)
+                        + seriennummer
+            - seriennummer: uuid
+            - token wird angelegt ist aber nicht aktiv!!! (counter == 0)
 
 
-    2. https://linotpserver/admin/init?
-        type=ocra&
-        genkey=1&
-        activationcode=AKTIVIERUNGSCODE&
-        user=BENUTZERNAME&
-        message=MESSAGE&
-        session=SESSIONKEY
+        2. https://linotpserver/admin/init?
+            type=ocra&
+            genkey=1&
+            activationcode=AKTIVIERUNGSCODE&
+            user=BENUTZERNAME&
+            message=MESSAGE&
+            session=SESSIONKEY
 
-        =>> "serial" : SERIENNUMMER, "nonce" : DATAOBJECT, "transactionid" : "TRANSAKTIONSID, "app_import" : IMPORTURL
+            =>> "serial" : SERIENNUMMER, "nonce" : DATAOBJECT, "transactionid" : "TRANSAKTIONSID, "app_import" : IMPORTURL
 
-        - nonce - von HSM oder random ?
-        - pkcs5 - kdf2
-        - es darf zur einer Zeit nur eine QR Token inaktiv (== im Ausrollzustand) sein !!!!!
-          der Token wird über den User gefunden
-        - seed = pdkdf2(nonce + activcode + shared secret)
-        - challenge generiern - von urandom oder HSM
-
-    3. check_t
-        - counter ist > nach der ersten Transaktion
-        - if counter >= 1: delete sharedsecret löschen
-
-
-    '''
+            - nonce - von HSM oder random ?
+            - pkcs5 - kdf2
+            - es darf zur einer Zeit nur eine QR Token inaktiv (== im Ausrollzustand) sein !!!!!
+            der Token wird über den User gefunden
+            - seed = pdkdf2(nonce + activcode + shared secret)
+            - challenge generiern - von urandom oder HSM
 
+        3. check_t
+            - counter ist > nach der ersten Transaktion
+            - if counter >= 1: delete sharedsecret löschen
+    """
 
 
     @classmethod
@@ -1627,7 +1670,7 @@ class OcraTokenClass(TokenClass):
             ocraSuite.compute(c_data)
 
         except Exception as ex:
-            log.error("[OcraTokenClass] %r" % (traceback.format_exc()))
+            log.exception("[OcraTokenClass]")
             raise Exception('[OcraTokenClass] Failed to create ocrasuite challenge: %r' % (ex))
 
         ##  save the object
@@ -1660,17 +1703,15 @@ class OcraTokenClass(TokenClass):
 
         except Exception as ex:
             ##  this might happen if we have a db problem or the uniqnes constrain does not fit
-            log.error("[OcraTokenClass] %r" % (traceback.format_exc()))
+            log.exception("[OcraTokenClass]")
             raise Exception('[OcraTokenClass] Failed to create challenge object: %s' % (ex))
 
-        realm = None
-        realms = self.token.getRealms()
-        if len(realms) > 0:
-            realm = realms[0]
+        realms = []
+        tokenrealms = self.token.getRealms()
+        for realm in tokenrealms:
+            realms.append(realm.name)
 
-        url = ''
-        if realm is not None:
-            url = get_qrtan_url(realm.name)
+        url = get_qrtan_url(realms)
 
         log.debug('[challenge]: %r: %r: %r' % (transid, challenge, url))
         return (transid, challenge, True, url)
@@ -1817,7 +1858,7 @@ class OcraTokenClass(TokenClass):
             elif "false" == async.lower():
                 autosync = False
         except Exception as ex:
-            log.error('autosync check undefined %r' % (ex))
+            log.exception('autosync check undefined %r' % (ex))
             return res
 
         ' if autosync is not enabled: do nothing '
@@ -1862,7 +1903,7 @@ class OcraTokenClass(TokenClass):
                 otp0 = passw
                 count_0 = ocraSuite.checkOtp(otp0, counter, syncWindow, challenge, pin=ocraPin, timeshift=timeShift)
             except Exception as ex:
-                log.error(' error during autosync0 %r' % (ex))
+                log.exception(' error during autosync0 %r' % (ex))
 
             if count_0 != -1:
                 tinfo['lChallenge'] = {'otpc' : count_0}
@@ -1880,7 +1921,7 @@ class OcraTokenClass(TokenClass):
                 otp1 = passw
                 count_1 = ocraSuite.checkOtp(otp1, counter, syncWindow, challenge, pin=ocraPin, timeshift=timeShift)
             except Exception as ex:
-                log.error(' error during autosync1 %r' % (ex))
+                log.exception(' error during autosync1 %r' % (ex))
 
             if count_1 == -1:
                 del tinfo['lChallenge']
@@ -1975,8 +2016,7 @@ class OcraTokenClass(TokenClass):
                 log.info('rollout for token %r not completed' % (self.getSerial()))
 
         except Exception as ex:
-            log.error('[OcraTokenClass:statusValidationFail] Error during validation finalisation for token %r :%r' % (self.getSerial(), ex))
-            log.error("[OcraTokenClass:statusValidationFail] %r" % (traceback.format_exc()))
+            log.exception('[OcraTokenClass:statusValidationFail] Error during validation finalisation for token %r :%r' % (self.getSerial(), ex))
             raise Exception(ex)
 
         finally:
@@ -2126,7 +2166,7 @@ class OcraTokenClass(TokenClass):
                             ret = True
 
         except Exception as ex:
-            log.error('[OcraTokenClass:resync] unknown error: %r' % (ex))
+            log.exception('[OcraTokenClass:resync] unknown error: %r' % (ex))
             raise Exception('[OcraTokenClass:resync] unknown error: %s' % (ex))
 
         log.debug('[resync]: %r ' % (ret))
@@ -2264,7 +2304,7 @@ class OcraTokenClass(TokenClass):
         try:
             ones = int(maxChallDef)
         except ValueError as ex:
-            log.error('[OcraTokrenClass:maxChallengeJanitor] Faild to convert OcraMaxChallenges value from config: %r :%r' % (maxChallDef, ex))
+            log.exception('[OcraTokrenClass:maxChallengeJanitor] Faild to convert OcraMaxChallenges value from config: %r :%r' % (maxChallDef, ex))
             ones = 3
 
         if ones <= 0:
@@ -2403,19 +2443,29 @@ class OcraTokenClass(TokenClass):
         response_detail = {}
 
         info = self.getInfo()
+        # add : app_import, serial and sharedsecret
         response_detail.update(info)
 
         otpkey = None
         if 'otpkey' in info:
             otpkey = info.get('otpkey')
-        response_detail["otpkey"] = otpkey
+
+        if otpkey != None:
+            response_detail["otpkey"] = {
+                        "order"      : '1',
+                        "description": _("OTP seed"),
+                        "value"      :  "seed://%s" % otpkey,
+                        "img"        :  create_img(otpkey, width=200),
+                        }
 
         ocra_url = info.get('app_import')
+
         response_detail["ocraurl"] = {
-               "description": _("URL for OCRA token"),
-               "value": ocra_url,
-                "img": create_img(ocra_url, width=250),
-               }
+                    "order"      : '0',
+                    "description": _("URL for OCRA token"),
+                    "value": ocra_url,
+                    "img": create_img(ocra_url, width=250),
+                   }
 
         return response_detail
 
