@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -23,19 +23,91 @@
 #    Contact: www.linotp.org
 #    Support: www.lsexperts.de
 #
+
 """This file containes the standard token definitions:
               - OCRA2TokenClass
 
    the OCRA2 Token will use the standard challenge response
    instead of the dedicated ocra/request and ocra/check_t
 
+    rollout an ocra2 token:
+    ------------------------
+    The rollout is a 2 step process with the following steps:
+
+    1. call the /admin/init controller with the following parameters
+
+      :param type: must be "ocra2" for an ocra2 token
+      :param genkey: must be "1", if the server should generate the seed
+                     otherwise you can use the :param otpkey: with the seed
+      :param ocrasuite: your ocra suite of choice e.g.
+                              "OCRA-1:HOTP-SHA256-8:C-QA64"
+      :param sharedsecret: value must be "1"
+      :param serial: optional, if the serial will be defined by external
+
+    as reply a set of information is returned, where the relevant part is the
+    in the image data for the softtoken qrscan in the structure
+         detail/ocraurl/img
+
+    <img    width=250   src="data:image/png;base64,iVBO....
+
+    which could be embedded in the enrollment application. Other relevant
+    information (as well part in the qr encoded data) is the
+
+    "sharedsecret": "25676ef34bd1873834bbe10c4c4176b0a9689619"
+
+    which is the server data part for the pairing process transferd to the
+    qrtan app.
+
+    2. complete the rollout
+    The qrtan app will calculate an activation code, that must be transfered
+    back to the server as a set of input data for the second enrollment step by
+    calling the /admin/init controller with the following parameters:
+
+      :param type: must be of the same token type "ocra2"
+      :param serial: must be the same as received from the first request
+      :param genkey: must be of "1", which indicates, that the init is not
+                     finished
+      :param activationcode: "GEZDGNBVGY3TQOJQ01",
+      :param ocrasuite: same ocrasuite as above
+                              "OCRA-1:HOTP-SHA256-8:C-QA64",
+      :param message: optional the message, that is displayed in the app, e.g.
+                      "Transaktion: Ausrollen eines OCRA2 Tokens",
+
+    As response again an <img > is returned, which is the 'finishing'
+    transaction, where the qrtan app will reply only with an otp value
+
+
+    Further challenge request and response processing could then be managed by
+    using the /validate/check_s with
+
+     :param serial: token serial number, as defined above
+     :param challenge: the challenge input data as heart of the transaction
+
+    or when using /validate/check with
+
+     :param user: the assigned token user / owner
+     :param passw: which contains the token pin
+     :param challenge: the challenge input data as heart of the transaction
+
+    a response to this request will then contain the /detail/ocraurl/img
+    image data and the transaction id, which is the referer to the incomming
+    challenge respones from the qrtan app.
+
+    The challenge response then is verified by /validate/check_t and the
+    parameters:
+
+     :param transactionid: the transaction id "440364804594",
+     :param pass: the otp value e.g. "48344099"
+
+    But as well the /validate/check controller could be used to verify the
+    transaction by providing in addition the user name.
+
 """
 
 import binascii
 import logging
 import time
 import datetime
-
 import traceback
 
 from linotp.lib.config  import getFromConfig
@@ -44,7 +116,8 @@ from linotp.lib.crypt   import decryptPin, encryptPin
 from linotp.lib.crypt   import kdf2
 from linotp.lib.crypt   import createNonce
 
-from linotp.lib.policy  import get_qrtan_url
+from linotp.lib.policy  import getPolicy
+from linotp.lib.policy  import getPolicyActionValue
 
 
 ### TODO: move this as ocra specific methods
@@ -53,7 +126,7 @@ from linotp.lib.util import normalize_activation_code
 
 from linotp.lib.ocra    import OcraSuite
 
-from linotp.lib.validate import create_challenge
+from linotp.lib.validate import create_challenge, check_pin, split_pin_otp
 from linotp.lib.validate import get_challenges
 from linotp.lib.reply   import create_img
 
@@ -75,9 +148,95 @@ optional = True
 required = False
 
 log = logging.getLogger(__name__)
+
+
+def qrtan_url(realms, callback_id=None):
+    """
+    Returns the URL for the half automatic mode for the QR TAN token
+    for the given realm
+
+    :remark: there might be more than one url, if the token
+             belongs to more than one realm
+
+    :param realms: list of realms or None
+
+    :return: url string
+
+    """
+    url = get_qrtan_url('qrtanurl', realms, callback_id=callback_id)
+    return url
+
+
+def qrtanurl_init(realms, callback_id=None):
+    '''
+    Returns the URL for the half automatic mode for the QR TAN token
+    for the given realm
+
+    :remark: there might be more than one url, if the token
+             belongs to more than one realm
+
+    :param realms: list of realms or None
+
+    :return: url string
+
+    '''
+    url = get_qrtan_url('qrtanurl_init', realms, callback_id=callback_id)
+    return url
+
+
+def get_qrtan_url(qrtan_policy_name, realms, callback_id=None):
+    '''
+    Worker to returns the URL for the half automatic mode for the QR TAN token
+    for the given realm
+
+    :param qrtan_policy_name: either 'qrtanurl_init' or 'qrtanurl'
+    :param realms: list of realms or None
+    :param callback_id: support of multiple callback definitions
+
+    :return: url string
+
+
+    :remark: there might be more than one url, if the token belongs to more
+             than one realm. it is tested, if all are the same, otherwise
+             an exception is raised
+
+    '''
+    log.debug("getting qrtan callback url ")
+    urls = []
+
+    # Policies defintions with wildcard defintions '*'
+    # if there is no realm defined, we can catch by this trick the
+    # policy definition, which have an realm wildcard definition '*'
+    # so that the wildcard will match as well the empty realm
+    # By setting the realm to '/:no realm:/' there is no collission with
+    # any realm as this string contains characters, which are not allowed in
+    # realm names
+    if realms is None or len(realms) == 0:
+        realms = ['/:no realm:/']
+
+    for realm in realms:
+        pol = getPolicy({"scope": "authentication", 'realm': realm})
+        url = getPolicyActionValue(pol, qrtan_policy_name, is_string=True,
+                                   subkey=callback_id)
+        if url:
+            urls.append(url)
+
+    # now verify, that all urls are the same
+    if len(urls) > 1:
+        for url in urls:
+            if url != url[0]:
+                raise Exception('multiple enrollement urls %r found for '
+                                'realm set: %r' % (urls, realms))
+
+    url = ''
+    if urls:
+        url = urls[0]
+
+    log.debug("got callback url %s for realms %r" % (url, realms))
+    return url
+
+
 #### Ocra2TokenClass #####################################
-
-
 class Ocra2TokenClass(TokenClass):
     '''
     Ocra2TokenClass  implement an ocra compliant token
@@ -165,17 +324,22 @@ class Ocra2TokenClass(TokenClass):
         if activationcode is not None:
             ## dont create a new key
             genkey = None
-            serial = getRolloutToken4User(user=user, serial=serial, tok_type=tok_type)
+            serial = getRolloutToken4User(user=user, serial=serial,
+                                          tok_type=tok_type)
             if serial is None:
-                raise Exception('no token found for user: %r or serial: %r' % (user, serial))
+                raise Exception('no token found for user: %r or serial: %r'
+                                % (user, serial))
             helper_param['serial'] = serial
-            helper_param['activationcode'] = normalize_activation_code(activationcode)
+            helper_param['activationcode'] = normalize_activation_code(
+                                                                activationcode)
 
         if ocrasuite is None:
             if sharedsecret is not None or  activationcode is not None:
-                ocrasuite = getFromConfig("QrOcraDefaultSuite", 'OCRA-1:HOTP-SHA256-6:C-QA64')
+                ocrasuite = getFromConfig("QrOcraDefaultSuite",
+                                          'OCRA-1:HOTP-SHA256-6:C-QA64')
             else:
-                ocrasuite = getFromConfig("OcraDefaultSuite", 'OCRA-1:HOTP-SHA256-8:C-QN08')
+                ocrasuite = getFromConfig("OcraDefaultSuite",
+                                          'OCRA-1:HOTP-SHA256-8:C-QN08')
             helper_param['ocrasuite'] = ocrasuite
 
         if genkey is not None:
@@ -189,10 +353,6 @@ class Ocra2TokenClass(TokenClass):
 
         return helper_param
 
-
-
-
-
     @classmethod
     def getClassInfo(cls, key=None, ret='all'):
         '''
@@ -248,12 +408,13 @@ class Ocra2TokenClass(TokenClass):
                                     },
 
                                   },
-
-            'policy' : { 'selfservice' : {'activate_OCRA2' : {'type' : 'bool'} }}
-
+            'policy': {'selfservice': {
+                            'activate_OCRA2': {'type': 'bool'}
+                            },  # eof selfservice
+                      }  # eof policy
         }
 
-        if key is not None and res.has_key(key):
+        if key and key in res:
             ret = res.get(key)
         else:
             if ret == 'all':
@@ -261,8 +422,6 @@ class Ocra2TokenClass(TokenClass):
 
         return ret
 
-
-
     def __init__(self, aToken):
         '''
         getInfo - return the status of the token rollout
@@ -290,21 +449,18 @@ class Ocra2TokenClass(TokenClass):
         log.debug('[getInfo] %r ' % (self.info))
         return self.info
 
-
     def update(self, params, reset_failcount=True):
         '''
         update: add further definition for token from param in case of init
         '''
         log.debug('[update] %r: %r: ' % (params, reset_failcount))
 
-
-        if params.has_key('ocrasuite'):
+        if 'ocrasuite' in params:
             self.ocraSuite = params.get('ocrasuite')
         else:
             activationcode = params.get('activationcode', None)
             sharedSecret = params.get('sharedsecret', None)
 
-
             if activationcode is None and sharedSecret is None:
                 self.ocraSuite = self.getOcraSuiteSuite()
             else:
@@ -313,12 +469,11 @@ class Ocra2TokenClass(TokenClass):
         if params.get('activationcode', None):
             ## due to changes in the tokenclass parameter handling
             ## we have to add for compatibility a genkey parameter
-            if params.has_key('otpkey') == False and params.has_key('genkey') == False:
+            if 'otpkey' not in params and 'genkey' not in params:
                 log.warning('[Ocra2TokenClass:update] missing parameter genkey\
                              to complete the rollout 2!')
                 params['genkey'] = 1
 
-
         TokenClass.update(self, params, reset_failcount=reset_failcount)
 
         self.addToTokenInfo('ocrasuite', self.ocraSuite)
@@ -331,7 +486,7 @@ class Ocra2TokenClass(TokenClass):
         if ocraPin is not None:
             self.token.setUserPin(ocraPin)
 
-        if params.has_key('otpkey'):
+        if 'otpkey' in params:
             self.setOtpKey(params.get('otpkey'))
 
         self._rollout_1(params)
@@ -340,8 +495,6 @@ class Ocra2TokenClass(TokenClass):
         log.debug('[update]:')
         return
 
-
-
     def _rollout_1(self, params):
         '''
         do the rollout 1 step
@@ -388,6 +541,15 @@ class Ocra2TokenClass(TokenClass):
             info['serial'] = self.getSerial()
             uInfo['se'] = self.getSerial()
 
+            callback = self._prepare_callback_url(params, qrtanurl_init)
+            callback = callback.encode('utf-8')
+
+            if callback:
+                uInfo['u'] = callback
+
+            # the info url must be provided in any case
+            info["url"] = callback
+
             info['app_import'] = 'lseqr://init?%s' % (urllib.urlencode(uInfo))
             del info['ocrasuite']
             self.info = info
@@ -397,6 +559,59 @@ class Ocra2TokenClass(TokenClass):
         log.debug('[_rollout_1]:')
         return
 
+    def _prepare_callback_url(self, params, policy_lookup_funtion,
+                               transactionid=None):
+        """
+        prepare the callback url
+        - check if it is allowed to get the callback from the parameters
+        - get callback url from parameters or as fallback, from policy value
+
+        finaly replace <user>, <password>, <transactionid>, <serial> in the url
+
+        :param params: the dict of calling parameters
+        :param policy_lookup_funtion: function to check for the policy defined
+                    callback, either standard callback or rollout callback
+        :param transactionid: optional the transactionid, if not in rollout
+                              scope
+        :return: the callback url or an empty string
+
+        """
+        callback = ''
+
+        realms = []
+        tokenrealms = self.token.getRealms()
+        for realm in tokenrealms:
+            realms.append(realm.name)
+
+        # is there an callbac selector
+        callback_id = params.get('callback.id', None)
+        callback = policy_lookup_funtion(realms, callback_id)
+
+        # is the callback supressed for the current request?
+        if 'no_callback' in params:
+            callback = ''
+
+        # now adjust the callback with replacements
+        if callback:
+            callback = callback.replace('<serial>', self.getSerial())
+
+            if '<transactionid>' in callback and transactionid:
+                callback = callback.replace('<transactionid>', transactionid)
+
+            # now handle the replacement parts for the authetication
+            callback_pass = params.get('callback.password', '')
+            callback_user = params.get('callback.user', '')
+
+            if "<user>" in callback and callback_user:
+                user = urllib.quote(callback_user)
+                callback = callback.replace('<user>', user)
+
+            if "<password>" in callback and callback_pass:
+                passw = urllib.quote(callback_pass)
+                callback = callback.replace('<password>', passw)
+
+        return callback
+
     def _rollout_2(self, params):
         '''
         2.
@@ -427,7 +642,8 @@ class Ocra2TokenClass(TokenClass):
             ##  genkey might have created a new key, so we have to rely on
             encSharedSecret = self.getFromTokenInfo('sharedSecret', None)
             if encSharedSecret is None:
-                raise Exception ('missing shared secret of initialition for token %r' % (self.getSerial()))
+                raise Exception('missing shared secret of initialition for '
+                                 'token %r' % (self.getSerial()))
 
             sharedSecret = decryptPin(encSharedSecret)
 
@@ -442,7 +658,6 @@ class Ocra2TokenClass(TokenClass):
             elif self.ocraSuite.find('-SHA512'):
                 key_len = 64
 
-
             newkey = kdf2(sharedSecret, nonce, activationcode, key_len)
             self.setOtpKey(binascii.hexlify(newkey))
 
@@ -455,8 +670,10 @@ class Ocra2TokenClass(TokenClass):
             (res, opt) = create_challenge(self, options=params)
 
             challenge = opt.get('challenge')
-            url = opt.get('url')
             transid = opt.get('transactionid')
+            url = opt.get('url')
+            url = url.replace('<serial>', self.getSerial())
+            url = url.replace('<transactionid>', transid)
 
             ##  generate response
             info = {}
@@ -472,17 +689,19 @@ class Ocra2TokenClass(TokenClass):
             if message is not None:
                 uInfo['me'] = str(message.encode("utf-8"))
 
-            ustr = urllib.urlencode({'u':str(url.encode("utf-8"))})
-            uInfo['u'] = ustr[2:]
-            info['url'] = str(url.encode("utf-8"))
+            ustr = urllib.urlencode({'u': str(url.encode("utf-8"))})
+            if ustr[2:]:
+                uInfo['u'] = ustr[2:]
+                info['url'] = str(url.encode("utf-8"))
 
             app_import = 'lseqr://nonce?%s' % (urllib.urlencode(uInfo))
 
             ##  add a signature of the url
-            signature = {'si': self.signData(app_import) }
+            signature = {'si': self.signData(app_import)}
             info['signature'] = signature.get('si')
 
-            info['app_import'] = "%s&%s" % (app_import, urllib.urlencode(signature))
+            info['app_import'] = "%s&%s" % (app_import,
+                                             urllib.urlencode(signature))
             self.info = info
 
             ##  setup new state
@@ -501,7 +720,8 @@ class Ocra2TokenClass(TokenClass):
         '''
         log.debug('[getOcraSuiteSuite]')
 
-        defaultOcraSuite = getFromConfig("OcraDefaultSuite", 'OCRA-1:HOTP-SHA256-8:C-QN08')
+        defaultOcraSuite = getFromConfig("OcraDefaultSuite",
+                                         'OCRA-1:HOTP-SHA256-8:C-QN08')
         self.ocraSuite = self.getFromTokenInfo('ocrasuite', defaultOcraSuite)
 
         log.debug('[getOcraSuiteSuite] %r:' % (self.ocraSuite))
@@ -516,14 +736,13 @@ class Ocra2TokenClass(TokenClass):
         '''
         log.debug('[getQROcraSuiteSuite]')
 
-        defaultOcraSuite = getFromConfig("QrOcraDefaultSuite", 'OCRA-1:HOTP-SHA256-8:C-QA64')
+        defaultOcraSuite = getFromConfig("QrOcraDefaultSuite",
+                                         'OCRA-1:HOTP-SHA256-8:C-QA64')
         self.ocraSuite = self.getFromTokenInfo('ocrasuite', defaultOcraSuite)
 
         log.debug('[getQROcraSuiteSuite] %r:' % (self.ocraSuite))
         return self.ocraSuite
 
-
-
     def signData(self, data):
         '''
         sign the received data with the secret key
@@ -580,9 +799,8 @@ class Ocra2TokenClass(TokenClass):
             ocraSuite.compute(c_data)
 
         except Exception as ex:
-            log.error("[Ocra2TokenClass] challenge verification failed: "
+            log.exception("[Ocra2TokenClass] challenge verification failed: "
                                 "%s,%r: " % (challenge, ex))
-            log.error("[Ocra2TokenClass] %r" % (traceback.format_exc()))
             ret = False
 
         return ret
@@ -596,60 +814,62 @@ class Ocra2TokenClass(TokenClass):
         ## which kind of challenge gen should be used
         typ = 'raw'
 
-        input = None
+        input_data = None
         challenge = None
         session = None
         message = ""
 
-        if options is not None:
-            input = options.get('challenge', None)
-            if input is None:
-                input = options.get('message', None)
-            if input is None:
-                input = options.get('data', None)
+        if options:
+            input_data = options.get('challenge',
+                                     options.get('message',
+                                                 options.get('data', None)))
 
             typ = options.get('challenge_type', 'raw')
             ## ocra token could contain a session attribute
             session = options.get('ocra_session', None)
 
-        if input is None or len(input) == 0:
+        if input_data is None or len(input_data) == 0:
             typ = 'random'
 
         secretHOtp = self.token.getHOtpKey()
         ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)
 
         if typ == 'raw':
-            challenge = ocraSuite.data2rawChallenge(input)
+            challenge = ocraSuite.data2rawChallenge(input_data)
         elif typ == 'random':
-            challenge = ocraSuite.data2randomChallenge(input)
+            challenge = ocraSuite.data2randomChallenge(input_data)
         elif typ == 'hash':
-            challenge = ocraSuite.data2hashChallenge(input)
+            challenge = ocraSuite.data2hashChallenge(input_data)
 
         log.debug('[Ocra2TokenClass] challenge: %r ' % (challenge))
 
         store_data = {
-                'challenge' : "%s" % (challenge),
-                'serial' : self.token.getSerial(),
-                'input' : '',
-                'url' : '',
+                'challenge': "%s" % (challenge),
+                'serial': self.token.getSerial(),
+                'input': '',
+                'url': '',
                 }
 
-        if input is not None:
-            store_data['input'] = input
+        if input_data is not None:
+            store_data['input'] = input_data
 
         if session is not None:
             store_data["session"] = session
 
         res = self.verify_challenge_is_valid(challenge, session)
 
-        ## add Info: so depending on the Info, the rendering could be done
-        ##          as a callback into the token via
-        ##          token.getQRImageData(opt=details)
-        realms = self.token.getRealms()
-        if len(realms) > 0:
-            store_data["url"] = get_qrtan_url(realms[0].name)
+        # add Info: so depending on the Info, the rendering could be done
+        #   as a callback into the token via
+        #       token.getQRImageData(opt=details)
 
-        ## we will return a dict of all
+        # do we have a callback url, that will receive the otp value
+        callback = self._prepare_callback_url(options, qrtan_url,
+                                          transactionid=state)
+        callback = callback.encode('utf-8')
+
+        store_data["url"] = callback
+
+        # we will return a dict of all
         attributes = self.prepare_message(store_data, state)
         attributes['challenge'] = challenge
 
@@ -675,14 +895,16 @@ class Ocra2TokenClass(TokenClass):
         u = urllib.urlencode({'u': "%s" % (url.encode("utf-8"))})
 
         challenge = data.get('challenge')
-        input = data.get('input')
+        input_data = data.get('input')
 
         uInfo = {'tr': transId,
                  'ch': challenge,
-                 'me': str(input.encode("utf-8")),
-                 'u': str(u[2:])}
-        detail = {'request'         : str(input.encode("utf-8")),
-                  'url'             : str(url.encode("utf-8")),
+                 'me': str(input_data.encode("utf-8")),
+                 }
+        if url:
+            uInfo['u'] = str(u[2:])
+        detail = {'request': str(input_data.encode("utf-8")),
+                  'url': str(url.encode("utf-8")),
                  }
 
         ## create the app_url from the data
@@ -694,6 +916,10 @@ class Ocra2TokenClass(TokenClass):
         dataobj = '%s&%s' % (dataobj, str(urllib.urlencode(signature)))
 
         detail["data"] = dataobj
+        detail["ocraurl"] = {
+                    "value": detail.get('data'),
+                    "img": create_img(detail.get('data'), width=250)
+                    }
 
         return detail
 
@@ -765,7 +991,6 @@ class Ocra2TokenClass(TokenClass):
             ocraSuite.compute(c_data)
 
         except Exception as ex:
-            log.error("[Ocra2TokenClass] %r" % (traceback.format_exc()))
             raise Exception('[Ocra2TokenClass] Failed to create ocrasuite '
                                                         'challenge: %r' % (ex))
 
@@ -784,14 +1009,12 @@ class Ocra2TokenClass(TokenClass):
             raise Exception('[Ocra2TokenClass] Failed to create '
                                                 'challenge object: %s' % (ex))
 
-        realm = None
-        realms = self.token.getRealms()
-        if len(realms) > 0:
-            realm = realms[0]
+        realms = []
+        tokenrealms = self.token.getRealms()
+        for realm in tokenrealms:
+            realms.append(realm.name)
 
-        url = ''
-        if realm is not None:
-            url = get_qrtan_url(realm.name)
+        url = qrtan_url(realms)
 
         log.debug('[challenge]: %r: %r: %r' % (transid, challenge, url))
         return (transid, challenge, True, url)
@@ -827,10 +1050,14 @@ class Ocra2TokenClass(TokenClass):
 
         return request_is_valid
 
-    def is_challenge_response(self, passw, user, options=None,
-                                                            challenges=None):
+    def is_challenge_response(self, passw, user, options=None, challenges=None):
         '''
-        for the ocra token,
+        test for the ocra token, if this is a response to a challenge
+
+        normal challenge response brings in a password and there is at least
+        a stored challenge available. But OCRA support as well direct
+        challenges, which bring the challenge data and the otp within the same
+        request.
 
         :param passw: password, which might be pin or pin+otp
         :param user: the requesting user
@@ -840,16 +1067,27 @@ class Ocra2TokenClass(TokenClass):
         '''
 
         challenge_response = False
-        if (passw is not None and  len(passw) > 0 and
-           challenges is not None and len(challenges) > 0):
-            challenge_response = True
 
-        if 'challenge' in options or 'data' in options:
+        if passw is not None and len(passw) > 0:
+            # for a challenge response a pin+otp is required
+            # if passw matches a password, this is a challenge request
+            if check_pin(self, passw, user, options=options):
+                return False
+
+            if challenges is not None and len(challenges) > 0:
+                challenge_response = True
+
+            # we might have a direct challenge:
+            # direct challenge comes along with a pin+otp and direct challenge
+            elif 'challenge' in options or 'data' in options:
+                    challenge_response = True
+
+        elif 'challenge' in options or 'data' in options:
             challenge_response = False
 
-        ## we leave out the checkOtp, which is done later
-        ## either in checkResponse4Challenge
-        ## or in the check pin+otp
+        # we leave out the checkOtp, which is done later
+        # either in checkResponse4Challenge
+        # or in the check pin+otp
 
         return challenge_response
 
@@ -888,9 +1126,20 @@ class Ocra2TokenClass(TokenClass):
         if 'session' in loptions:
             del loptions['session']
 
-        (res, pin, otpval) = self.splitPinPass(passw)
+        (pin, otpval) = self.splitPinPass(passw)
         res = self.checkPin(pin)
 
+        if res == False:
+            if 'transactionid' in options or 'state' in options:
+                transactionid = options.get('state', options.get('transactionid'))
+                for challenge in challenges:
+                    transid = challenge.get('transid', None)
+                    if transid == transactionid:
+                        res = True
+                        pin = None
+                        otpval = passw
+                        break
+
         if res == True:
             window = self.getCounterWindow()
             counter = self.getOtpCount()
@@ -919,6 +1168,12 @@ class Ocra2TokenClass(TokenClass):
                     matching_challenges.append(mids.get(transid))
                     break
 
+            # direct challenge -
+            # brings the challange along with the matching pin
+            if not mids and 'challenge' in options:
+                otpcount = self.checkOtp(otpval, counter, window,
+                                         options=options)
+
         return (otpcount, matching_challenges)
 
     def checkOtp(self, passw , counter, window, options=None):
@@ -1076,7 +1331,7 @@ class Ocra2TokenClass(TokenClass):
             elif "false" == async.lower():
                 autosync = False
         except Exception as ex:
-            log.error('autosync check undefined %r' % (ex))
+            log.exception('autosync check undefined %r' % (ex))
             return res
 
         ' if autosync is not enabled: do nothing '
@@ -1113,7 +1368,7 @@ class Ocra2TokenClass(TokenClass):
         ## autosync does only work, if we have a token info, where the last challenge and the last sync-counter is stored
         ## if no tokeninfo, we start with a autosync request, thus start the lookup in the sync window
 
-        if tinfo.has_key('lChallenge') == False:
+        if 'lChallenge' not in tinfo:
             ## run checkOtp, with sync window for the current challenge
             log.info('[OcraToken:autosync] initial sync')
             count_0 = -1
@@ -1121,7 +1376,7 @@ class Ocra2TokenClass(TokenClass):
                 otp0 = passw
                 count_0 = ocraSuite.checkOtp(otp0, counter, syncWindow, challenge, pin=ocraPin, timeshift=timeShift)
             except Exception as ex:
-                log.error(' error during autosync0 %r' % (ex))
+                log.exception(' error during autosync0 %r' % (ex))
 
             if count_0 != -1:
                 tinfo['lChallenge'] = {'otpc' : count_0}
@@ -1139,7 +1394,7 @@ class Ocra2TokenClass(TokenClass):
                 otp1 = passw
                 count_1 = ocraSuite.checkOtp(otp1, counter, syncWindow, challenge, pin=ocraPin, timeshift=timeShift)
             except Exception as ex:
-                log.error(' error during autosync1 %r' % (ex))
+                log.exception(' error during autosync1 %r' % (ex))
 
             if count_1 == -1:
                 del tinfo['lChallenge']
@@ -1180,10 +1435,6 @@ class Ocra2TokenClass(TokenClass):
         log.debug('[autosync]: %r ' % (res))
         return res
 
-
-
-
-
     def statusValidationFail(self):
         '''
         statusValidationFail - callback to enable a status change,
@@ -1196,7 +1447,7 @@ class Ocra2TokenClass(TokenClass):
         log.debug('[statusValidationFail]')
         challenge = None
 
-        if self.transId == 0 :
+        if self.transId == 0:
             return
         try:
 
@@ -1205,24 +1456,27 @@ class Ocra2TokenClass(TokenClass):
                 challenge = challenges[0]
                 challenge.setTanStatus(received=True, valid=False)
 
-
             ##  still in rollout state??
             rolloutState = self.getFromTokenInfo('rollout', '0')
 
             if rolloutState == '1':
-                log.info('rollout state 1 for token %r not completed' % (self.getSerial()))
+                log.info('rollout state 1 for token %r not completed'
+                         % (self.getSerial()))
 
             elif rolloutState == '2':
                 if challenge.received_count >= int(getFromConfig("OcraMaxChallengeRequests", '3')):
                     ##  after 3 fails in rollout state 2 - reset to rescan
                     self.addToTokenInfo('rollout', '1')
-                    log.info('rollout for token %r reset to phase 1:' % (self.getSerial()))
+                    log.info('rollout for token %r reset to phase 1:'
+                             % (self.getSerial()))
 
-                log.info('rollout for token %r not completed' % (self.getSerial()))
+                log.info('rollout for token %r not completed'
+                         % (self.getSerial()))
 
         except Exception as ex:
-            log.error('[Ocra2TokenClass:statusValidationFail] Error during validation finalisation for token %r :%r' % (self.getSerial(), ex))
-            log.error("[Ocra2TokenClass:statusValidationFail] %r" % (traceback.format_exc()))
+            log.exception('[Ocra2TokenClass:statusValidationFail] Error during '
+                          'validation finalisation for token %r :%r'
+                          % (self.getSerial(), ex))
             raise Exception(ex)
 
         finally:
@@ -1232,7 +1486,6 @@ class Ocra2TokenClass(TokenClass):
         log.debug('[statusValidationFail]')
         return
 
-
     def statusValidationSuccess(self):
         '''
         statusValidationSuccess - callback to enable a status change,
@@ -1244,7 +1497,7 @@ class Ocra2TokenClass(TokenClass):
         '''
         log.debug('[statusValidationSuccess]')
 
-        if self.transId == 0 :
+        if self.transId == 0:
             return
 
         challenges = get_challenges(self.getSerial(), transid=self.transId)
@@ -1258,11 +1511,11 @@ class Ocra2TokenClass(TokenClass):
 
         if rolloutState == '2':
             t_info = self.getTokenInfo()
-            if t_info.has_key('rollout'):
+            if 'rollout' in t_info:
                 del t_info['rollout']
-            if t_info.has_key('sharedSecret'):
+            if 'sharedSecret' in t_info:
                 del t_info['sharedSecret']
-            if t_info.has_key('nonce'):
+            if 'nonce' in t_info:
                 del t_info['nonce']
             self.setTokenInfo(t_info)
 
@@ -1305,7 +1558,6 @@ class Ocra2TokenClass(TokenClass):
             challenge1['session'] = ch1.get('session')
             challenge1['id'] = ch1.get('id')
 
-
             ## the elder one
             ch2 = challenges[0]
             challenge2['challenge'] = ch2.get('data').get('challenge')
@@ -1314,19 +1566,16 @@ class Ocra2TokenClass(TokenClass):
             challenge2['id'] = ch2.get('id')
 
         else:
-            if options.has_key('challenge1'):
+            if 'challenge1' in options:
                 challenge1['challenge'] = options.get('challenge1')
-            if options.has_key('challenge2'):
+            if 'challenge2' in options:
                 challenge2['challenge'] = options.get('challenge2')
 
-
         if len(challenge1) == 0 or len(challenge2) == 0:
             error = "No challeges found!"
             log.error('[Ocra2TokenClass:resync] %s' % (error))
             raise Exception('[Ocra2TokenClass:resync] %s' % (error))
 
-
-
         secretHOtp = self.token.getHOtpKey()
         ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)
 
@@ -1379,7 +1628,7 @@ class Ocra2TokenClass(TokenClass):
                             ret = True
 
         except Exception as ex:
-            log.error('[Ocra2TokenClass:resync] unknown error: %r' % (ex))
+            log.exception('[Ocra2TokenClass:resync] unknown error: %r' % (ex))
             raise Exception('[Ocra2TokenClass:resync] unknown error: %s' % (ex))
 
         log.debug('[resync]: %r ' % (ret))
@@ -1431,13 +1680,29 @@ class Ocra2TokenClass(TokenClass):
         response_detail = {}
 
         info = self.getInfo()
+        # add : app_import, serial and sharedsecret
         response_detail.update(info)
 
+        otpkey = None
+        if 'otpkey' in info:
+            otpkey = info.get('otpkey')
+
+        if otpkey != None:
+            response_detail["otpkey"] = {
+                        "order"      : '1',
+                        "description": _("OTP seed"),
+                        "value"      :  "seed://%s" % otpkey,
+                        "img"        :  create_img(otpkey, width=200),
+                        }
+
         ocra_url = info.get('app_import')
+
         response_detail["ocraurl"] = {
-               "description" : _("URL for OCRA token"),
-               "value" : ocra_url,
-               "img"   : create_img(ocra_url, width=250)}
+                    "order"      : '0',
+                    "description" : _("URL for OCRA2 token"),
+                    "value" : ocra_url,
+                    "img"   : create_img(ocra_url, width=250)
+                    }
 
         return response_detail
 
