@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -27,8 +27,6 @@
 
 import logging
 import copy
-import traceback
-
 
 import httplib2
 import urllib
@@ -81,6 +79,9 @@ class RemoteTokenClass(TokenClass):
         self.remoteResConf = None
         self.mode = ['authenticate', 'challenge']
 
+        self.isRemoteChallengeRequest = False
+        self.local_pin_check = None
+
     @classmethod
     def getClassType(cls):
         """
@@ -197,11 +198,13 @@ class RemoteTokenClass(TokenClass):
         local_check = False
         if 1 == int(self.getFromTokenInfo("remote.local_checkpin")):
             local_check = True
-        log.debug(" local checking pin? %r" % local_check)
+
+        # preserve this info for later uasge
+        self.local_pin_check = local_check
+        log.debug(" local checking PIN? %r" % local_check)
 
         return local_check
 
-
     def authenticate(self, passw, user, options=None):
         """
         do the authentication on base of password / otp and user and
@@ -229,7 +232,7 @@ class RemoteTokenClass(TokenClass):
             (res, pin, otpval) = split_pin_otp(self, passw, user,
                                                options=options)
 
-            res = TokenClass.checkPin(self, pin)
+            res = check_pin(self, pin, user=user, options=options)
             if res is False:
                 return (res, otp_counter, reply)
 
@@ -259,9 +262,12 @@ class RemoteTokenClass(TokenClass):
             if pin_match is True:
                 request_is_valid = True
 
+        elif self.isRemoteChallengeRequest:
+            request_is_valid = True
+
         return request_is_valid
 
-    def do_request(self, passw, transactionid=None, user=None):
+    def do_request(self, passw, transactionid=None, user=None, autoassign=False):
         """
         run the http request against the remote host
 
@@ -272,8 +278,6 @@ class RemoteTokenClass(TokenClass):
 
         :return: Tuple of (success, otp_count= -1 or 0, reply=remote response)
         """
-
-        reply = {}
         otpval = passw.encode("utf-8")
 
         remoteServer = self.getFromTokenInfo("remote.server") or ""
@@ -312,10 +316,18 @@ class RemoteTokenClass(TokenClass):
                   (len(otpval), remoteServer, remoteSerial, remoteUser))
         params = {}
 
-        if len(remoteSerial) > 0:
+        if autoassign:
+            params['user'] = user.login
+            params['realm'] = remoteRealm
+            user.realm = remoteRealm
+            if len(remotePath) == 0:
+                remotePath = "/validate/check"
+
+        elif len(remoteSerial) > 0:
             params['serial'] = remoteSerial
             if len(remotePath) == 0:
                 remotePath = "/validate/check_s"
+
         elif len(remoteUser) > 0:
             params['user'] = remoteUser
             params['realm'] = remoteRealm
@@ -336,14 +348,18 @@ class RemoteTokenClass(TokenClass):
         if transactionid is not None:
             params['state'] = transactionid
 
-        ## use a POST request to check the token
+        # use a POST request to check the token
         data = urllib.urlencode(params)
-        request_url = "%s%s" % (remoteServer, remotePath)
+        request_url = "%s/%s" % (remoteServer.rstrip('/'), remotePath.lstrip('/'))
+
+        reply = {}
+        otp_count = -1
+        res = False
 
         try:
             ## prepare the submit and receive headers
             headers = {"Content-type": "application/x-www-form-urlencoded",
-                       "Accept": "text/plain"}
+                       "Accept": "text/plain", 'Connection': 'close'}
 
             ## submit the request
             try:
@@ -378,14 +394,33 @@ class RemoteTokenClass(TokenClass):
                 reply = copy.deepcopy(result["detail"])
                 otp_count = -1
                 res = False
+                self.isRemoteChallengeRequest = True
+                self.remote_challenge_response = reply
 
         except Exception as exx:
-            log.error("[do_request] [RemoteToken] Error getting response from "
+            log.exception("[do_request] [RemoteToken] Error getting response from "
                       "remote Server (%r): %r" % (request_url, exx))
-            log.error("[do_request] %r" % (traceback.format_exc()))
 
         return (res, otp_count, reply)
 
+    def createChallenge(self, transactionid, options=None):
+        """
+        for every remote challenge we have to create a local challenge
+        e.g. to support multiple challenges
+
+        remark: we might call the super of this method first
+        """
+
+        message = 'Remote Otp: '
+        data = {'serial': self.token.getSerial()}
+
+        if self.isRemoteChallengeRequest:
+            data['remote_reply'] = self.remote_challenge_response
+
+        attributes = None
+
+        return (True, message, data, attributes)
+
     def checkResponse4Challenge(self, user, passw, options=None,
                                 challenges=None):
         '''
@@ -424,30 +459,51 @@ class RemoteTokenClass(TokenClass):
             transid = options.get('transactionid', options.get('state', None))
 
         if transid is not None:
+            # lookup if there is a transaction with this transaction id
+            for challenge in challenges:
+                if challenge.transid == transid:
+                    matching_challenge = challenge
+                    break
+            if matching_challenge is None:
+                return (otp_counter, matching_challenges)
+
             ## in case of a local pin check, we the transaction is a local one
             ## and we must not forward this!!
             if self.check_pin_local():
                 ## check if transaction id is in list of challengens
-                for challenge in challenges:
-                    if challenge.transid == transid:
-                        matching_challenge = challenge
-                        break
-
-                if matching_challenge is not None:
-                    (res, otp_counter, reply) = \
-                        self.do_request(passw, user=user)
-
-                    ## everything is ok, we remove the challenge
-                    if res is True and otp_counter >= 0:
-                        matching_challenges.append(matching_challenge)
-
-            ## in case of remote check pin, we just forward everything
-            else:
                 (res, otp_counter, reply) = \
-                    self.do_request(passw, transactionid=transid, user=user)
+                    self.do_request(passw, user=user)
+
+                ## everything is ok, we remove the challenge
+                if res is True and otp_counter >= 0:
+                    matching_challenges.append(matching_challenge)
+
+            # in case of remote check pin, we lookup in the local challenge
+            # to extract the replyed challenge
+            else:
+                remote_transid = matching_challenge.get('data', {}).\
+                                            get('remote_reply', {}).\
+                                            get('transactionid', '')
+                (res, otp_counter, reply) = \
+                    self.do_request(passw, transactionid=remote_transid,
+                                    user=user)
+                ## everything is ok, we remove the challenge
+                if res and otp_counter >= 0:
+                    matching_challenges.append(matching_challenge)
 
         return (otp_counter, matching_challenges)
 
+    def check_otp_exist(self, otp, window=None, user=None, autoassign=False):
+        '''
+        checks if the given OTP value is/are values of this very token.
+        This is used to autoassign and to determine the serial number of
+        a token.
+        '''
+        (res, otp_count, reply) = self.do_request(otp, user=user,
+                                                  autoassign=autoassign)
+        return otp_count
+
+
     def checkPin(self, pin, options=None):
         """
         check the pin - either remote or localy
@@ -471,19 +527,16 @@ class RemoteTokenClass(TokenClass):
         :return: tupple of the (success, pin and otpvalue)
 
         """
-        res = 0
-
         local_check = self.check_pin_local()
 
-
         if local_check:
-            (res, pin, otpval) = TokenClass.splitPinPass(self, passw)
+            (pin, otpval) = TokenClass.splitPinPass(self, passw)
         else:
             pin = ""
             otpval = passw
 
         log.debug("[splitPinPass] [remotetoken] returnung (len:%r) (len:%r)"
                   % (len(pin), len(otpval)))
-        return (res, pin, otpval)
+        return pin, otpval
 
 ###eof#########################################################################
