@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -25,6 +25,78 @@
 #
 """This file containes the dynamic sms token implementation:
               - SMSTokenClass (sms)
+
+    the SMS Token is an challenge - response token, by the means, that there is
+    a first request, which triggers the challenge (=sending of sms message)
+    and a second request, which refers to the initial request by the
+    transactionid and verifies the otp value:
+
+    /validate/check_s
+
+    with params
+     :param serial: the token serialnumber [required]
+     :param pass: the token pin or
+                  if the token belongs to an user, the user pin/password
+                  (s. otppin policy) [required]
+    :param data: the message, that will contain the otp value [optional]
+                 * In the message, the strings <otp>, <serial> and
+                   <transactionid> will be replaced
+                 * if no data is provided, the smstext - policy value will be
+                   evaluated. Fallback is the message "<otp>"
+    :param message: alternative name for the data parameter
+
+    :return: json result wit tansaction an message, that will be displayed to
+             the user
+
+                {
+                    "detail": {
+                        "transactionid": "172682842808",
+                        "message": "sms submitted",
+                        "state": "172682842808"
+                    },
+                    "version": "LinOTP 2.7.2.1",
+                    "jsonrpc": "2.0",
+                    "result": {
+                        "status": true,
+                        "value": false
+                    },
+                    "id": 0
+
+                }
+
+    for the validation of the sms request now the controller method
+
+    /validate/check_t
+
+    could be used with the parameters
+
+    :param pass: received otp value
+    :param transactionid: the transactionid, which referes, that the pin
+                          has been verified and checked
+
+    alternativly the controller method
+
+    /validate/check_s
+
+    could be used as well, by providing the combination of the pin+otp
+    in the pass parameter:
+
+    :param serial: serial number of the token
+    :param pass: the password consisting of fixed part and the otp part
+
+    :return: json response
+
+    {
+        "version": "LinOTP 2.7.2.1",
+        "jsonrpc": "2.0",
+        "result": {
+            "status": true,
+            "value": true
+        },
+        "id": 0
+    }
+
+
 """
 
 import time
@@ -35,6 +107,8 @@ from linotp.lib.HMAC    import HmacOtp
 from linotp.lib.util    import getParam
 from linotp.lib.util    import required
 
+from linotp.lib.user import getUserDetail
+
 from linotp.lib.validate import check_pin
 from linotp.lib.validate import check_otp
 from linotp.lib.validate import split_pin_otp
@@ -46,13 +120,13 @@ from linotp.lib.policy import getPolicyActionValue
 from linotp.lib.policy import getPolicy
 from linotp.lib.policy import get_auth_AutoSMSPolicy
 
+
 import sys
 if sys.version_info[0:2] >= (2, 6):
     from json import loads, dumps
 else:
     from simplejson import loads, dumps
 
-
 from pylons.i18n.translation import _
 
 from linotp.lib.tokens.hmactoken import HmacTokenClass
@@ -67,17 +141,13 @@ except ImportError as exx:
     log.warning("Failed to import SMSProvider %s" % exx)
     SMSPROVIDER_IMPORTED = False
 
-
-keylen = {'sha1'    : 20,
-          'sha256'  : 32,
-          'sha512'  : 64,
+keylen = {'sha1': 20,
+          'sha256': 32,
+          'sha512': 64,
           }
 
 
-
 ##################################################################
-
-
 def get_auth_smstext(user="", realm=""):
     '''
     this function checks the policy scope=authentication, action=smstext
@@ -90,16 +160,41 @@ def get_auth_smstext(user="", realm=""):
     ret = False
     smstext = "<otp>"
 
-    pol = getPolicy({'scope': 'authentication', 'realm': realm,
-                      "action" : "smstext" })
+    pol = getPolicy({'scope': 'authentication',
+                     'realm': realm,
+                     "action": "smstext"})
 
     if len(pol) > 0:
-        smstext = getPolicyActionValue(pol, "smstext", String=True)
+        smstext = getPolicyActionValue(pol, "smstext", is_string=True)
         log.debug("[get_auth_smstext] got the smstext = %s" % smstext)
         ret = True
 
     return ret, smstext
 
+
+def is_phone_editable(user=""):
+    '''
+    this function checks the policy scope=selfservice, action=edit_sms
+    This is a int policy, while the '0' is a deny
+    '''
+    # the default string is the OTP value
+    ret = True
+    realm = user.realm
+    login = user.login
+
+    policies = getPolicy({'scope': 'selfservice',
+                          'realm': realm,
+                          "action": "edit_sms",
+                          "user": login},
+                          )
+    if policies:
+        edit_sms = getPolicyActionValue(policies, "edit_sms")
+        if edit_sms == 0:
+            ret = False
+
+    return ret
+
+
 class SmsTokenClass(HmacTokenClass):
     '''
     implementation of the sms token class
@@ -169,12 +264,16 @@ class SmsTokenClass(HmacTokenClass):
                                      },
                                     },
                                   },
-
-
-
+            'policy': {'selfservice':
+                       {'edit_sms':
+                        {'type': 'int',
+                         'value': [0, 1],
+                         'desc': _('define if the user should be allowed'
+                                    ' to define the sms')
+                         }}}
         }
 
-        if key is not None and res.has_key(key):
+        if key and key in res:
             ret = res.get(key)
         else:
             if ret == 'all':
@@ -182,7 +281,6 @@ class SmsTokenClass(HmacTokenClass):
 
         return ret
 
-
     def update(self, param, reset_failcount=True):
         '''
         update - process initialization parameters
@@ -198,11 +296,24 @@ class SmsTokenClass(HmacTokenClass):
 
         # specific - phone
         phone = getParam(param, "phone", required)
+
+        # in scope selfservice - check if edit_sms is allowed
+        # if not allowed to edit, check if the phone is the same
+        # as from the user data
+        if param.get('::scope::', {}).get('selfservice', False):
+            user = param['::scope::']['user']
+            if not is_phone_editable(user):
+                u_info = getUserDetail(user)
+                u_phone = u_info.get('mobile', u_info.get('phone', None))
+                if u_phone != phone:
+                    raise Exception(_('User is not allowed to '
+                                      'set phone number'))
+
         self.setPhone(phone)
 
-        ## in case of the sms token, only the server must know the otpkey
-        ## thus if none is provided, we let create one (in the TokenClass)
-        if not param.has_key('genkey') and not param.has_key('otpkey'):
+        # in case of the sms token, only the server must know the otpkey
+        # thus if none is provided, we let create one (in the TokenClass)
+        if 'genkey' not in param and 'otpkey' not in param:
             param['genkey'] = 1
 
         HmacTokenClass.update(self, param, reset_failcount)
@@ -210,7 +321,8 @@ class SmsTokenClass(HmacTokenClass):
         log.debug("[update] end. all token parameters are set.")
         return
 
-    def is_challenge_response(self, passw, user, options=None, challenges=None):
+    def is_challenge_response(self, passw, user, options=None,
+                              challenges=None):
         '''
         check, if the request contains the result of a challenge
 
@@ -221,24 +333,25 @@ class SmsTokenClass(HmacTokenClass):
         :return: returns true or false
         '''
 
-        challenge_response = False
-
         if "state" in options or "transactionid" in options:
-            challenge_response = True
+            return True
 
-        ## it as well might be a challenge response,
-        ## if the passw is longer than the pin
-        if challenge_response == False:
-            (res, pin, otpval) = split_pin_otp(self, passw, user=user,
-                                                            options=options)
-            if res >= 0 :
-                res = check_pin(self, pin, user=user, options=options)
-                if res == True and len(otpval) > 0:
-                    challenge_response = True
+        # it as well might be a challenge response,
+        # if the passw is longer than the pin
+        (res, pin, otpval) = split_pin_otp(self, passw, user=user,
+                                                        options=options)
+        if res >= 0:
+            otp_counter = check_otp(self, otpval, options=options)
+            if otp_counter >= 1:
+                pin_match = check_pin(self, pin, user=user, options=options)
+                if not pin_match:
+                    return False
+            if otp_counter >= 0:
+                return True
 
-        return challenge_response
+        return False
 
-### challenge interfaces starts here
+# ## challenge interfaces starts here
     def is_challenge_request(self, passw, user, options=None):
         '''
         check, if the request would start a challenge
@@ -255,18 +368,16 @@ class SmsTokenClass(HmacTokenClass):
         '''
 
         request_is_valid = False
-        ## do we need to call the
-        #(res, pin, otpval) = split_pin_otp(self, passw, user, options=options)
-
+        # if its a challenge, the passw contains only the pin
         pin_match = check_pin(self, passw, user=user, options=options)
         if pin_match is True:
             request_is_valid = True
 
         return request_is_valid
 
-    ##
-    ##!!! this function is to be called in the sms controller !!!
-    ##
+    #
+    #!!! this function is to be called in the sms controller !!!
+    #
     def submitChallenge(self, options=None):
         '''
         submit the sms message - former method name was checkPin
@@ -277,27 +388,46 @@ class SmsTokenClass(HmacTokenClass):
         '''
         res = 0
         user = None
+        result = _("sending sms failed")
         message = "<otp>"
 
-        ## it is configurable, if sms should be triggered by a valid pin
+        if not options:
+            options = {}
+
+        # it is configurable, if sms should be triggered by a valid pin
         send_by_PIN = getFromConfig("sms.sendByPin") or True
 
-        if self.isActive() == True and send_by_PIN == True :
+        if self.isActive() == True and send_by_PIN == True:
             counter = self.getOtpCount()
             log.debug("[submitChallenge] counter=%r" % counter)
-            self.incOtpCounter(counter, reset=False)
-            # At this point we must not bail out in case of an
-            # Gateway error, since checkPIN is successful. A bail
+
+            # At this point we MUST NOT bail out in case of an
+            # Gateway error, since checkPIN is successful, as the bail
             # out would cancel the checking of the other tokens
             try:
+                realms = self.getRealms()
+                if realms:
+                    sms_ret, new_message = get_auth_smstext(realm=realms[0])
+                    if sms_ret:
+                        message = new_message
+
+                user = options.get('user', '')
+                if user:
+                    sms_ret, new_message = get_auth_smstext(
+                                            realm=user.realm)
+                    if sms_ret:
+                        message = new_message
+
+                if 'data' in options or 'message' in options:
+                    message = options.get('data',
+                                          options.get('message', '<otp>'))
+
+                transactionid = options.get('transactionid', None)
+                res, result = self.sendSMS(message=message,
+                                              transactionid=transactionid)
 
-                if options is not None and type(options) == dict:
-                    user = options.get('user', None)
-                    if user:
-                        _sms_ret, message = get_auth_smstext(
-                                                realm=user.realm)
-                res, message = self.sendSMS(message=message)
                 self.info['info'] = "SMS sent: %r" % res
+                log.debug('SMS sent: %s', result)
 
             except Exception as e:
                 # The PIN was correct, but the SMS could not be sent.
@@ -306,12 +436,14 @@ class SmsTokenClass(HmacTokenClass):
                           " SMS could not be sent: %r" % e)
                 log.warning("[submitChallenge] %s" % info)
                 res = False
-                message = info
-        if len(message) == 0:
-            pass
+                result = info
 
-        return res, message
+            finally:
+                # we increment the otp in any case, independend if sending
+                # of the sms was sucsessful
+                self.incOtpCounter(counter, reset=False)
 
+        return res, result
 
     def initChallenge(self, transactionid, challenges=None, options=None):
         """
@@ -342,13 +474,13 @@ class SmsTokenClass(HmacTokenClass):
         for challenge in challenges:
             start = challenge.get('timestamp')
             expiry = start + datetime.timedelta(seconds=blocking_time)
-            ## check if there is already a challenge underway
+            # # check if there is already a challenge underway
             if now >= start and now <= expiry:
                 transid = challenge.getTransactionId()
                 message = 'sms with otp already submitted'
                 success = False
-                attributes = {'info' : 'challenge already submitted',
-                              'state' : transid}
+                attributes = {'info': 'challenge already submitted',
+                              'state': transid}
                 break
 
         return (success, transid, message, attributes)
@@ -369,29 +501,29 @@ class SmsTokenClass(HmacTokenClass):
         success = False
         sms = ""
         message = ""
-        attributes = {'state':transactionid}
+        attributes = {'state': transactionid}
 
+        options['state'] = transactionid
         success, sms = self.submitChallenge(options=options)
 
         if success is True:
             message = 'sms submitted'
             self.setValidUntil()
         else:
-            attributes = {'state' : ''}
+            attributes = {'state': ''}
             message = 'sending sms failed'
-            if len('sms') > 0:
+            if sms:
                 message = sms
 
-        ## after submit set validity time in readable
-        ## datetime format in the storeing data
+        # after submit set validity time in readable
+        # datetime format in the storeing data
         timeScope = self.loadLinOtpSMSValidTime()
         expiryDate = datetime.datetime.now() + \
                                     datetime.timedelta(seconds=timeScope)
-        data = {'valid_until' : "%s" % expiryDate }
+        data = {'valid_until': "%s" % expiryDate}
 
         return (success, message, data, attributes)
 
-
     def checkResponse4Challenge(self, user, passw, options=None, challenges=None):
         """
         verify the response of a previous challenge
@@ -419,7 +551,7 @@ class SmsTokenClass(HmacTokenClass):
 
         otp_val = passw
 
-        ## fallback: do we have pin+otp ??
+        # # fallback: do we have pin+otp ??
         (res, pin, otp) = split_pin_otp(self, passw, user=user,
                                                             options=options)
 
@@ -431,7 +563,7 @@ class SmsTokenClass(HmacTokenClass):
         for challenge in challenges:
             otp_count = self.checkOtp(otp_val, counter, window,
                                                             options=options)
-            if otp_count > 0 :
+            if otp_count > 0:
                 matching.append(challenge)
                 break
 
@@ -449,26 +581,44 @@ class SmsTokenClass(HmacTokenClass):
         :rtype: int
         '''
 
-        log.debug("[checkOtp] begin. start to verify the otp value: anOtpVal:" +
+        log.debug("[checkOtp] begin. start to verify the otp value: anOtpVal:"
                   " %r, counter: %r, window: %r, options: %r "
                   % (anOtpVal, counter, window, options))
 
+        if not options:
+            options = {}
+
         ret = HmacTokenClass.checkOtp(self, anOtpVal, counter, window)
         if ret != -1:
             if self.isValid() == False:
                 ret = -1
 
         if ret >= 0:
+            # checko if autosms is defined
+            # if so, autamatical send an new otp after successful login
             if get_auth_AutoSMSPolicy():
                 user = None
                 message = "<otp>"
-                if options is not None and type(options) == dict:
+                realms = self.getRealms()
+                if realms:
+                    _sms_ret, message = get_auth_smstext(realm=realms[0])
+
+                if 'user' in options:
                     user = options.get('user', None)
                     if user:
-                        sms_ret, message = get_auth_smstext(realm=user.realm)
-                self.incOtpCounter(ret, reset=False)
-                success, message = self.sendSMS(message=message)
+                        _sms_ret, message = get_auth_smstext(realm=user.realm)
+                realms = self.getRealms()
 
+                if 'data' in options or 'message' in options:
+                    message = options.get('data',
+                                          options.get('message', '<otp>'))
+
+                try:
+                    _success, message = self.sendSMS(message=message)
+                except Exception as exx:
+                    log.exception(exx)
+                finally:
+                    self.incOtpCounter(ret, reset=False)
         if ret >= 0:
             msg = "otp verification was successful!"
         else:
@@ -476,7 +626,6 @@ class SmsTokenClass(HmacTokenClass):
         log.debug("[checkOtp] end. %s ret: %r" % (msg, ret))
         return ret
 
-
     def getNextOtp(self):
         '''
         access the nex validf otp
@@ -487,26 +636,25 @@ class SmsTokenClass(HmacTokenClass):
         log.debug("[getNextOtp] begin. starting to look for the next otp")
 
         try:
-            ### TODO - replace tokenLen
+            # ## TODO - replace tokenLen
             otplen = int(self.token.LinOtpOtpLen)
         except ValueError as ex:
-            log.error("[getNextOtp] ValueError %r" % ex)
+            log.exception("[getNextOtp] ValueError %r" % ex)
             raise Exception(ex)
 
         secret_obj = self.token.getHOtpKey()
         counter = self.token.getOtpCounter()
 
-        #log.debug("serial: %s",serialNum)
+        # log.debug("serial: %s",serialNum)
         hmac2otp = HmacOtp(secret_obj, counter, otplen)
         nextotp = hmac2otp.generate(counter + 1)
 
-
         log.debug("[getNextOtp] end. got the next otp value: nextOtp %r"
                                                                     % nextotp)
         return nextotp
 
-    ### in the SMS token we use the generic TokenInfo
-    ### to store the phone number
+    # in the SMS token we use the generic TokenInfo
+    # to store the phone number
     def setPhone(self, phone):
         '''
         setter for the phone number
@@ -554,10 +702,10 @@ class SmsTokenClass(HmacTokenClass):
 
         (_phone, until) = self.getSMSInfo()
 
-        ## suport for direct verification
+        # # suport for direct verification
         if until == 0:
             timeScope = self.loadLinOtpSMSValidTime()
-            until = int (time.time()) + timeScope
+            until = int(time.time()) + timeScope
         return until
 
     def setSMSInfo(self, key, value):
@@ -584,25 +732,17 @@ class SmsTokenClass(HmacTokenClass):
         :return: tuple of phone number and validity time in unix lifetime sec
         '''
         log.debug("[getSMSInfo] begin. get the sms token info")
-        phone = ""
-        until = 0
 
         info = self.getTokenInfo()
+        phone = info.get("phone", '')
+        until = info.get("until", 0)
 
-        if info.has_key("phone") == True:
-            phone = info.get("phone")
-
-        if info.has_key("until") == True:
-            until = info.get("until")
-
-        log.debug("[getSMSInfo] end. got the following token info: (phone: " +
+        log.debug("[getSMSInfo] end. got the following token info: (phone: "
                   " %r, until: %r)" % (phone, until))
         return (phone, until)
 
-
-    ### we take the countWindow.column to store the time
-    ##  in int format (cut off the .2 from the time() )
-    ###
+    # we take the countWindow.column to store the time
+    #  in int format (cut off the .2 from the time() )
 
     def setValidUntil(self):
         '''
@@ -612,12 +752,12 @@ class SmsTokenClass(HmacTokenClass):
         '''
         log.debug("[setValidUntil] begin. set the due date")
         timeScope = self.loadLinOtpSMSValidTime()
-        dueDate = int (time.time()) + timeScope
+        dueDate = int(time.time()) + timeScope
         self.setUntil(dueDate)
-        #self.token.setCountWindow(dueDate)
+        # self.token.setCountWindow(dueDate)
 
-        log.debug("[setValidUntil] end. define the following due date:" +
-                                                      " dueDate %r" % (dueDate))
+        log.debug("[setValidUntil] end. define the following due date:"
+                                                    " dueDate %r" % (dueDate))
         return dueDate
 
     def isValid(self):
@@ -627,7 +767,7 @@ class SmsTokenClass(HmacTokenClass):
         :return: True or False
         :rtype: boolean
         '''
-        log.debug("[isValid] begin. check if challenge timeframe " +
+        log.debug("[isValid] begin. check if challenge timeframe "
                                                               "is still valid")
         ret = False
         dueDate = self.getUntil()
@@ -641,7 +781,7 @@ class SmsTokenClass(HmacTokenClass):
         log.debug("[isValid] %s: ret: %r" % (msg, ret))
         return ret
 
-    def sendSMS(self, message="<otp>"):
+    def sendSMS(self, message=None, transactionid=None):
         '''
         send sms
 
@@ -653,11 +793,14 @@ class SmsTokenClass(HmacTokenClass):
         :rtype: string
 
         '''
-        log.debug("[sendSMS] begin. process the submitting of " +
+        log.debug("[sendSMS] begin. process the submitting of "
                                               "the sms message %r" % (message))
 
         ret = None
 
+        if not message:
+            message = "<otp>"
+
         if not SMSPROVIDER_IMPORTED:
             raise Exception("The SMSProvider could not be imported. Maybe you "
                             "didn't install the package (Debian "
@@ -667,9 +810,19 @@ class SmsTokenClass(HmacTokenClass):
         otp = self.getNextOtp()
         serial = self.getSerial()
 
+        if '<otp>' not in message:
+            log.error('Message unconfigured: prepending <otp> to message')
+            if isinstance(message, basestring):
+                message = "<otp> %s" % message
+            else:
+                message = "<otp> %r" % message
+
         message = message.replace("<otp>", otp)
         message = message.replace("<serial>", serial)
 
+        if transactionid:
+            message = message.replace("<transactionid>", transactionid)
+
         log.debug("[sendSMS] sending SMS to phone number %s " % phone)
         (SMSProvider, SMSProviderClass) = self.loadLinOtpSMSProvider()
         log.debug("[sendSMS] smsprovider: %s, class: %s"
@@ -678,26 +831,26 @@ class SmsTokenClass(HmacTokenClass):
         try:
             sms = getSMSProviderClass(SMSProvider, SMSProviderClass)()
         except Exception as exc:
-            log.error("[sendSMS] Failed to load SMSProvider: %r" % exc)
-            log.error("[sendSMS] %s" % traceback.format_exc())
+            log.exception("[sendSMS] Failed to load SMSProvider: %r" % exc)
             raise exc
 
         try:
-            ## now we need the config from the env
+            # # now we need the config from the env
             log.debug("[sendSMS] loading SMS configuration for class %s" % sms)
             config = self.loadLinOtpSMSProviderConfig()
             log.debug("[sendSMS] config: %r" % config)
             sms.loadConfig(config)
         except Exception as exc:
-            log.error("[sendSMS] Failed to load SMSProviderConfig: %r" % exc)
-            log.error("[sendSMS] %s" % traceback.format_exc())
+            log.exception("[sendSMS] Failed to load SMSProviderConfig: %r" % exc)
             raise Exception("Failed to load SMSProviderConfig: %r" % exc)
 
-        log.debug("[sendSMS] submitMessage: %r, to phone %r" % (message, phone))
+        log.debug("[sendSMS] submitMessage: %r, to phone %r", message, phone)
         ret = sms.submitMessage(phone, message)
+        if not ret:
+            raise Exception("Failed to submit message")
         log.debug("[sendSMS] message submitted")
 
-        ## after submit set validity time
+        # # after submit set validity time
         self.setValidUntil()
 
         # return OTP for selftest purposes
@@ -705,7 +858,6 @@ class SmsTokenClass(HmacTokenClass):
                                                                 % (message))
         return ret, message
 
-
     def loadLinOtpSMSProvider(self):
         '''
         get the SMS Provider class definition
@@ -713,23 +865,22 @@ class SmsTokenClass(HmacTokenClass):
         :return: tuple of SMSProvider and Provider Class as string
         :rtype: tuple of (string, string)
         '''
-        log.debug('[loadLinOtpSMSProvider] begin. get the SMS Provider ' +
-                                                             'class definition')
+        log.debug('[loadLinOtpSMSProvider] begin. get the SMS Provider '
+                                                            'class definition')
         smsProvider = getFromConfig("SMSProvider")
 
         if smsProvider is not None:
             (SMSProvider, SMSProviderClass) = smsProvider.rsplit(".", 1)
 
-        log.debug('[loadLinOtpSMSProvider] end. using the following ' +
+        log.debug('[loadLinOtpSMSProvider] end. using the following '
                    'SMS Provider class: (SMSProvider: %r,SMSProviderClass: %r)'
                   % (SMSProvider, SMSProviderClass))
         return (SMSProvider, SMSProviderClass)
 
-        #SMSProvider         = "smsprovider.HttpSMSProvider"
-        ##SMSProvider         = "HttpSMSProvider"
-        #SMSProviderClass    = "HttpSMSProvider"
-        #return (SMSProvider,SMSProviderClass)
-
+        # SMSProvider         = "smsprovider.HttpSMSProvider"
+        # #SMSProvider         = "HttpSMSProvider"
+        # SMSProviderClass    = "HttpSMSProvider"
+        # return (SMSProvider,SMSProviderClass)
 
     def loadLinOtpSMSProviderConfig(self):
         '''
@@ -738,17 +889,17 @@ class SmsTokenClass(HmacTokenClass):
         :return: dict of the sms provider definition
         :rtype: dict
         '''
-        log.debug('[loadLinOtpSMSProviderConfig] begin. load the sms ' +
+        log.debug('[loadLinOtpSMSProviderConfig] begin. load the sms '
                                                   'provider config definition')
 
-        ## get User realm
+        # # get User realm
         config = {}
 
         tConfig = getFromConfig("enclinotp.SMSProviderConfig", None)
         if tConfig is None:
             tConfig = getFromConfig("SMSProviderConfig", "{}")
 
-        ## fix the handling of multiline config entries
+        # # fix the handling of multiline config entries
         lconfig = []
         lines = tConfig.splitlines()
         for line in lines:
@@ -759,15 +910,18 @@ class SmsTokenClass(HmacTokenClass):
         tConfig = " ".join(lconfig)
         log.debug("[loadLinOtpSMSProviderConfig] providerconfig: %s"
                                                                     % tConfig)
+        try:
+            if tConfig is not None:
+                config = loads(tConfig)
+        except ValueError as exx:
+            raise ValueError('Failed to load provider config:%r %r'
+                             % (tConfig, exx))
 
-        if tConfig is not None:
-            config = loads(tConfig)
-
-        log.debug('[loadLinOtpSMSProviderConfig] sms provider config' +
+        log.debug('[loadLinOtpSMSProviderConfig] sms provider config'
                                                 ' found: config %r' % (config))
         return config
 
-        #config  = {'URL':'http://127.0.0.1:5001/validate/ok',
+        # config  = {'URL':'http://127.0.0.1:5001/validate/ok',
         #      'PARAMETER': {
         #                  'von':'OWN_NUMBER',
         #                  'passwort':'PASSWORD',
@@ -778,7 +932,7 @@ class SmsTokenClass(HmacTokenClass):
         #       'SMS_PHONENUMBER_KEY':'ziel',
         #       'HTTP_Method':'GET',
         #      }
-        #return config
+        # return config
 
     def loadLinOtpSMSValidTime(self):
         '''
@@ -799,5 +953,15 @@ class SmsTokenClass(HmacTokenClass):
                                                                    % (timeout))
         return timeout
 
-###eof#########################################################################
+    def getInitDetail(self, params, user=None):
+        '''
+        to complete the token normalisation, the response of the initialiastion
+        should be build by the token specific method, the getInitDetails
+        '''
+        response_detail = {}
 
+        response_detail['serial'] = self.getSerial()
+
+        return response_detail
+
+###eof#########################################################################
