@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -117,13 +117,13 @@ class TimeHmacTokenClass(HmacTokenClass):
         self.hKeyRequired = True
 
         ''' timeStep defines the granularity: '''
-        self.timeStep = getFromConfig("totp.timeStep", 30)
+        self.timeStep = getFromConfig("totp.timeStep", 30) or 30
 
         ''' window size in seconds:
             30 seconds with as step width of 30 seconds results
             in a window of 1  which is one attempt
         '''
-        self.timeWindow = getFromConfig("totp.timeWindow", 180)
+        self.timeWindow = getFromConfig("totp.timeWindow", 180) or 180
 
 
         '''the time shift is specified in seconds  - and could be
@@ -134,7 +134,7 @@ class TimeHmacTokenClass(HmacTokenClass):
         '''we support various hashlib methods, but only on create
         which is effectively set in the update
         '''
-        self.hashlibStr = getFromConfig("totp.hashlib", u'sha1')
+        self.hashlibStr = getFromConfig("totp.hashlib", u'sha1') or 'sha1'
 
         log.debug("[init]  end. Token object created")
         return
@@ -275,18 +275,21 @@ class TimeHmacTokenClass(HmacTokenClass):
         if val is not None:
             self.timeShift = val
 
-
         HmacTokenClass.update(self, param)
 
-        self.addToTokenInfo("timeWindow", self.timeWindow)
-        self.addToTokenInfo("timeShift", self.timeShift)
-        self.addToTokenInfo("timeStep", self.timeStep)
-        self.addToTokenInfo("hashlib", self.hashlibStr)
+        if self.timeWindow is not None and self.timeWindow != '':
+            self.addToTokenInfo("timeWindow", self.timeWindow)
+        if self.timeShift is not None and self.timeShift != '':
+            self.addToTokenInfo("timeShift", self.timeShift)
+        if self.timeStep is not None and self.timeStep != '':
+            self.addToTokenInfo("timeStep", self.timeStep)
+        if self.hashlibStr:
+            self.addToTokenInfo("hashlib", self.hashlibStr)
 
         log.debug("[update]  end. Processing the initialization parameters done.")
         return
 
-    def check_otp_exist(self, otp, window=10):
+    def check_otp_exist(self, otp, window=10, user=None, autoassign=False):
         '''
         checks if the given OTP value is/are values of this very token.
         This is used to autoassign and to determine the serial number of
@@ -358,8 +361,7 @@ class TimeHmacTokenClass(HmacTokenClass):
             try:
                 dt = datetime.datetime.strptime(curTime, tFormat)
             except Exception as ex:
-                log.error('[time2float] Error during conversion of datetime: %r' % (ex))
-                log.error("[time2float] %r" % traceback.format_exc())
+                log.exception('[time2float] Error during conversion of datetime: %r' % (ex))
                 raise Exception(ex)
         else:
             log.error("[time2float] invalid curTime: %s. You need to specify a datetime.datetime" % type(curTime))
@@ -407,11 +409,11 @@ class TimeHmacTokenClass(HmacTokenClass):
             raise e
 
         secretHOtp = self.token.getHOtpKey()
-        self.hashlibStr = self.getFromTokenInfo("hashlib", self.hashlibStr)
+        self.hashlibStr = self.getFromTokenInfo("hashlib", self.hashlibStr) or 'sha1'
 
-        timeStepping = int(self.getFromTokenInfo("timeStep", self.timeStep))
-        window = int(self.getFromTokenInfo("timeWindow", self.timeWindow))
-        shift = int(self.getFromTokenInfo("timeShift", self.timeShift))
+        timeStepping = int(self.getFromTokenInfo("timeStep", self.timeStep) or 30)
+        window = int(self.getFromTokenInfo("timeWindow", self.timeWindow) or 180)
+        shift = int(self.getFromTokenInfo("timeShift", self.timeShift) or 0)
 
         ## oldCounter we have to remove one, as the normal otp handling will increment
         oCount = self.getOtpCount() - 1
@@ -518,7 +520,7 @@ class TimeHmacTokenClass(HmacTokenClass):
             elif "false" == async.lower():
                 autosync = False
         except Exception as e:
-            log.error('autosync check failed %r' % e)
+            log.exception('autosync check failed %r' % e)
             return res
 
         ' if autosync is not enabled: do nothing '
@@ -688,12 +690,11 @@ class TimeHmacTokenClass(HmacTokenClass):
 
         res = (-1, 0, 0, 0)
 
-
         otplen = int(self.token.LinOtpOtpLen)
         secretHOtp = self.token.getHOtpKey()
-        self.hashlibStr = self.getFromTokenInfo("hashlib", "sha1")
-        timeStepping = int(self.getFromTokenInfo("timeStep", 30))
-        shift = int(self.getFromTokenInfo("timeShift", 0))
+        self.hashlibStr = self.getFromTokenInfo("hashlib", "sha1") or 'sha1'
+        timeStepping = int(self.getFromTokenInfo("timeStep", 30) or 30)
+        shift = int(self.getFromTokenInfo("timeShift", 0) or 0)
 
         hmac2Otp = HmacOtp(secretHOtp, self.getOtpCount(), otplen, self.getHashlib(self.hashlibStr))
 
@@ -738,9 +739,9 @@ class TimeHmacTokenClass(HmacTokenClass):
 
         secretHOtp = self.token.getHOtpKey()
 
-        self.hashlibStr = self.getFromTokenInfo("hashlib", "sha1")
-        timeStepping = int(self.getFromTokenInfo("timeStep", 30))
-        shift = int(self.getFromTokenInfo("timeShift", 0))
+        self.hashlibStr = self.getFromTokenInfo("hashlib", "sha1") or 'sha1'
+        timeStepping = int(self.getFromTokenInfo("timeStep", 30) or 30)
+        shift = int(self.getFromTokenInfo("timeShift", 0) or 0)
 
         hmac2Otp = HmacOtp(secretHOtp, self.getOtpCount(),
                            otplen, self.getHashlib(self.hashlibStr))
