@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -26,7 +26,6 @@
 """ contains user - related functions """
 import logging
 import re
-import traceback
 import sys
 
 from linotp.lib.error   import UserError
@@ -39,6 +38,8 @@ from linotp.lib.realm   import getRealms
 
 from linotp.lib.resolver import splitResolver
 from linotp.lib.resolver import getResolverObject
+from linotp.lib.resolver import getResolverClassName
+from linotp.lib.resolver import getResolverList
 
 
 from linotp.lib.realm import createDBRealm
@@ -59,6 +60,9 @@ class User(object):
         self.login = ""
         self.realm = ""
         self.conf = ""
+        self.info = {}
+        self.exist = False
+
 
         if login is not None:
             self.login = login
@@ -72,6 +76,74 @@ class User(object):
         self.resolverConf = {}
         self.resolvers_list = []
 
+    @staticmethod
+    def getUserObject(login, realm=None, check_if_exist=False):
+
+        f_realm = realm
+        f_login = login
+
+        if not realm:
+            if '@' in login:
+                realms = getRealms()
+                lo, rea = login.rsplit('@', 1)
+                if rea.lower() in realms:
+                    f_realm = rea.lower()
+                    f_login = lo
+                else:
+                    f_realm = getDefaultRealm()
+                    f_login = login
+
+        f_user = User(f_login, realm=f_realm)
+        if check_if_exist:
+            uid, resolver = f_user.get_uid_resolver()
+
+        return f_user
+
+    def does_exists(self, resolvers=None):
+        """
+        """
+        uid, _resolver = self.get_uid_resolver(resolvers=resolvers)
+        if uid is not None:
+            return True
+        return False
+
+    def get_uid_resolver(self, resolvers=None):
+        uid = None
+        resolver = None
+        resolvers_list = []
+
+        if not resolvers:
+            if self.realm:
+                realms = getRealms()
+                if self.realm.lower() in realms:
+                    resolvers_list = realms.get(self.realm.lower(), {}).\
+                                       get('useridresolver', [])
+        else:
+            resolvers_list = []
+            for search_resolver in resolvers:
+                fq_resolver = User.get_fq_resolver(search_resolver)
+                if fq_resolver:
+                    resolvers_list.append(fq_resolver)
+
+        if not resolvers_list:
+            return None, None
+
+        for resolver in resolvers_list:
+            try:
+                y = getResolverObject(resolver)
+                uid = y.getUserId(self.login)
+                if not uid:
+                    uid = None
+                    continue
+                self.resolverUid[resolver] = uid
+                self.exist = True
+                break
+
+            except Exception as exx:
+                log.exception("Error while accessing resolver %r", exx)
+
+        return (uid, resolver)
+
     def getRealm(self):
         return self.realm
 
@@ -82,25 +154,25 @@ class User(object):
         return self.login
 
     def isEmpty(self):
-        ## ignore if only conf is set! as it makes no sense
+        # ignore if only conf is set! as it makes no sense
         if len(self.login) + len(self.realm) == 0:
             return True
         else:
             return False
-##def __eq__(self,other):
-##    ret = False
-##    if other is None:
-##        if self.isEmpty() == True:
-##            return True
-##        else:
-##            return False
-##    if other.login == self.login and self.realm == other.realm and other.conf == self.conf:
-##        return True
-##    else:
-##        return False
+# def __eq__(self,other):
+#    ret = False
+#    if other is None:
+#        if self.isEmpty() == True:
+#            return True
+#        else:
+#            return False
+#    if other.login == self.login and self.realm == other.realm and other.conf == self.conf:
+#        return True
+#    else:
+#        return False
 
-##def __ne__(self,other):
-##      return not(self.__eq__(other))
+# def __ne__(self,other):
+#      return not(self.__eq__(other))
 
     def __str__(self):
         ret = str(None)
@@ -130,6 +202,44 @@ class User(object):
         """
         self.resolvers_list = resolvers
 
+    @staticmethod
+    def get_fq_resolver(res):
+        fq_resolver = None
+        resolvers = getResolverList()
+        if res in resolvers:
+            match_res = resolvers.get(res)
+            fq_resolver = getResolverClassName(match_res['type'],
+                                               match_res['resolvername'])
+        return fq_resolver
+
+    def getUserInfo(self, resolver=None):
+        userInfo = {}
+
+        lookup_resolvers = None
+        if resolver:
+            lookup_resolvers = [resolver]
+
+        userid, resolverC = self.get_uid_resolver(lookup_resolvers)
+
+        if not userid:
+            return {}
+
+        try:
+            (package, module, _class, _conf) = splitResolver(resolverC)
+            module = package + "." + module
+
+            y = getResolverObject(resolverC)
+            log.debug("[getUserInfo] Getting user info for userid "
+                      ">%r< in resolver" % userid)
+            userInfo = y.getUserInfo(userid)
+            self.info[resolverC] = userInfo
+
+        except Exception as e:
+            log.exception("[getUserInfo][ module %r notfound! :%r ]" % (module, e))
+
+        return userInfo
+
+
     def getResolvers(self):
         return self.resolverUid.keys()
 
@@ -144,6 +254,31 @@ class User(object):
 
         return uid
 
+    def getUserPerConf(self):
+        """
+        a wildcard usr (realm = *) could have multiple configurations
+        this method will return a list of uniq users, one per configuration
+
+        :return: list of users
+        """
+        resolvers = self.getResolvers()
+        if len(resolvers) == 1:
+            return [self]
+
+        # if we have multiple resolvers in this wildcard user
+        # we create one user per config and add this user to the list
+        # of all users to be checked
+        userlist = []
+        for resolver in resolvers:
+            (resId, resClass, resConf) = self.getResolverConf(resolver)
+            uid = self.getResolverUId(resolver)
+            n_user = User(self.login)
+            n_user.addResolverUId(resClass, uid, resConf, resId, resClass)
+            userlist.append(n_user)
+
+        return userlist
+
+
     def getResolverConf(self, resolver):
         conf = ""
         if self.resolverConf.has_key(resolver):
@@ -152,7 +287,7 @@ class User(object):
         return conf
 
 def getUserResolverId(user, report=False):
-    ## here we call the userid resolver!!"
+    # here we call the userid resolver!!"
     log.debug('getUserResolverId for %r' % user)
 
     (uuserid, uidResolver, uidResolverClass) = (u'', u'', u'')
@@ -161,29 +296,33 @@ def getUserResolverId(user, report=False):
         try:
             (uuserid, uidResolver, uidResolverClass) = getUserId(user)
         except Exception as e:
-            log.error('[getUserResolverId] for %r@%r failed: %r' % (user.login, user.realm, e))
-            log.error("[getUserResolverId] %s" % traceback.format_exc())
+            log.exception('[getUserResolverId] for %r@%r failed: %r' % (user.login, user.realm, e))
             if report == True:
                 raise UserError("getUserResolverId failed: %r" % e, id=1112)
 
     return (uuserid, uidResolver, uidResolverClass)
 
+
 def splitUser(username):
+    """
+    split the username into the user and realm
+
+    :param username: the given username
+    :return: tuple of (user and group/realm)
+    """
 
     user = username.strip()
     group = ""
 
-    ## todo split the last
-    l = user.split('@')
-    if len(l) >= 2:
-        (user, group) = user.rsplit('@')
-    else:
-        l = user.split('\\')
-        if len(l) >= 2:
-            (group, user) = user.rsplit('\\')
+
+    if '@' in user:
+        (user, group) = user.rsplit('@', 1)
+    elif '\\' in user:
+        (group, user) = user.split('\\', 1)
 
     return (user, group)
 
+
 def getUserFromParam(param, optionalOrRequired):
     realm = ""
     conf = ""
@@ -211,8 +350,8 @@ def getUserFromParam(param, optionalOrRequired):
 
     if param.has_key("resConf"):
         conf = param["resConf"]
-        ## with the short resolvernames, we have to extract the
-        ## configuration name from the resolver spec
+        # with the short resolvernames, we have to extract the
+        # configuration name from the resolver spec
         if "(" in conf and ")" in conf:
             res_conf, resolver_typ = conf.split(" ")
             conf = res_conf
@@ -283,7 +422,7 @@ def getUserFromRequest(request):
                        % d_auth)
 
     except Exception as e:
-        log.error("[getUserFromRequest] An error occurred when trying to fetch "
+        log.exception("[getUserFromRequest] An error occurred when trying to fetch "
                   "the user from the request: %r" % e)
         pass
 
@@ -308,7 +447,7 @@ def setRealm(realm, resolvers):
 
     createDBRealm(realm)
 
-    ## if this is the first one, make it the default
+    # if this is the first one, make it the default
     realms = getRealms()
     if 1 == len(realms):
         for name in realms:
@@ -344,6 +483,7 @@ def getUserRealms(user):
 
     return Realms
 
+
 def getRealmBox():
     '''
     returns the config value of selfservice.realmbox.
@@ -359,6 +499,17 @@ def getRealmBox():
     else:
         return False
 
+
+def getSplitAtSign():
+    '''
+    returns the config value of splitAtSign.
+    if True, the username should be split if there is an at sign.
+    if False, the username will be taken unchanged for loginname.
+    '''
+    splitAtSign = getFromConfig("splitAtSign", "true") or 'true'
+    return "true" == splitAtSign.lower()
+
+
 def getConf(Realms, Conf):
     """
     extract the configguration part from the resolver definition
@@ -421,7 +572,7 @@ def getResolvers(user):
 
     return Resolver
 
-def getResolversOfUser(user):
+def getResolversOfUser(user, use_default_realm=True):
     '''
     This returns the list of the Resolvers of a user in a given realm.
     Usually this should only return one resolver
@@ -441,10 +592,14 @@ def getResolversOfUser(user):
     if len(Resolvers) > 0:
         return Resolvers
 
-    if realm is None or realm == "":
-        realm = getDefaultRealm()
+    if not login:
+        return Resolvers
 
-    #if realm is None or realm=="" or login is None or login == "":
+    if realm is None or realm == "":
+        if use_default_realm:
+            realm = getDefaultRealm()
+
+    # if realm is None or realm=="" or login is None or login == "":
     #    log.error("[getResolversOfUser] You need to specify the name ( %s) and the realm (%s) of a user with conf %s" % (login, realm, user.conf))
 
     realms = getRealms();
@@ -480,10 +635,10 @@ def getResolversOfUser(user):
                     log.info("[getResolversOfUser] user %r found in resolver %r" % (login, realm_resolver))
                     log.info("[getResolversOfUser] userid resolved to %r " % uid)
 
-                    ## Unicode Madness:
-                    ## This will break as soon as the unicode "uid" is put into a tuple
-                    ## v = (login, realm_resolver, uid)
-                    ## log.info("[getResolversOfUser] %s %s %s" % v)
+                    # Unicode Madness:
+                    # This will break as soon as the unicode "uid" is put into a tuple
+                    # v = (login, realm_resolver, uid)
+                    # log.info("[getResolversOfUser] %s %s %s" % v)
                     resId = y.getResolverId();
                     resCId = realm_resolver
                     Resolvers.append(realm_resolver)
@@ -492,16 +647,15 @@ def getResolversOfUser(user):
                     log.debug("[getResolversOfUser] user %r not found"
                               " in resolver %r" % (login, realm_resolver))
             except Exception as e:
-                log.error("[getResolversOfUser] error searching user in"
+                log.exception("[getResolversOfUser] error searching user in"
                           " module %r:%r" % (module, e))
-                log.error("[getResolversOfUser] %s" % traceback.format_exc())
 
             log.debug("[getResolversOfUser] Resolvers: %r" % Resolvers)
 
     log.debug("[getResolversOfUser] Found the user %r in %r" % (login, Resolvers))
     return Resolvers
 
-def getUserId(user):
+def getUserId(user, resolvers=None):
     """
     getUserId (userObject)
 
@@ -512,7 +666,9 @@ def getUserId(user):
     loginUser = u''
     loginUser = user.login;
 
-    resolvers = getResolversOfUser(user)
+    if not resolvers:
+        resolvers = getResolversOfUser(user)
+
     for reso in resolvers:
         resId = ""
         resIdC = ""
@@ -528,7 +684,7 @@ def getUserId(user):
             if conf.lower() != user.conf.lower():
                 continue
 
-        ## try to load the UserIdResolver Class
+        # try to load the UserIdResolver Class
         try:
             module = package + "." + module
             log.debug("[getUserId] Getting resolver class: [%r] [%r]"
@@ -551,7 +707,7 @@ def getUserId(user):
                 break;
 
         except Exception as e:
-            log.error("[getUserId] module %r: %r ]" % (module, e))
+            log.exception("[getUserId] module %r: %r ]" % (module, e))
             continue
 
     if (uid == ''):
@@ -577,7 +733,7 @@ def getSearchFields(User):
             if conf.lower() != User.conf.lower():
                 continue
 
-        ## try to load the UserIdResolver Class
+        # try to load the UserIdResolver Class
         try:
             y = getResolverObject(reso)
             sf = y.getSearchFields()
@@ -589,15 +745,15 @@ def getSearchFields(User):
 
     return searchFields
 
-def getUserList(param, User):
+def getUserList(param, search_user):
 
     users = []
 
     searchDict = {}
     log.debug("[getUserList] entering function getUserList")
 
-    ## we have to recreate a new searchdict without the realm key
-    ## as delete does not work
+    # we have to recreate a new searchdict without the realm key
+    # as delete does not work
     for key in param:
         lval = param[key]
         if key == "realm":
@@ -608,45 +764,115 @@ def getUserList(param, User):
         searchDict[key] = lval
         log.debug("[getUserList] Parameter key:%r=%r" % (key, lval))
 
-    resolverrrs = getResolvers(User)
+    resolverrrs = getResolvers(search_user)
 
     for reso in resolverrrs:
-        (package, module, class_, conf) = splitResolver(reso)
+        (package, module, _class, conf) = splitResolver(reso)
         module = package + "." + module
 
-        if len(User.conf) > 0:
-            if conf.lower() != User.conf.lower():
+        if len(search_user.conf) > 0:
+            if conf.lower() != search_user.conf.lower():
                 continue
 
-        ## try to load the UserIdResolver Class
+        # try to load the UserIdResolver Class
         try:
+
             log.debug("[getUserList] Check for resolver class: %r" % reso)
             y = getResolverObject(reso)
-            log.debug("[getUserList] with this search dictionary: %r " % searchDict)
-            ulist = y.getUserList(searchDict)
-            log.debug("[getUserList] setting the resolver <%r> for each user" % reso)
-            for u in ulist:
-                u["useridresolver"] = reso
+            log.debug("[getUserList] with this search dictionary: %r "
+                      % searchDict)
 
-            log.debug("[getUserList] Found this userlist: %r" % ulist)
-            users.extend (ulist)
+            if hasattr(y, 'getUserListIterator'):
+                try:
+                    ulist_gen = y.getUserListIterator(searchDict)
+                    while True:
+                        ulist = ulist_gen.next()
+                        log.debug("[getUserList] setting the resolver <%r> "
+                                  "for each user" % reso)
+                        for u in ulist:
+                            u["useridresolver"] = reso
+                        log.debug("[getUserList] Found this userlist: %r"
+                                  % ulist)
+                        users.extend(ulist)
+
+                except StopIteration as exx:
+                    # we are done: all users are fetched or
+                    # page size limit reached
+                    pass
+            else:
+                ulist = y.getUserList(searchDict)
+                for u in ulist:
+                    u["useridresolver"] = reso
+                log.debug("[getUserList] Found this userlist: %r" % ulist)
+                users.extend(ulist)
 
         except KeyError as exx:
-            log.error("[getUserList][ module %r:%r ]" % (module, exx))
-            log.error("[getUserList] %s" % traceback.format_exc())
+            log.exception("[getUserList][ module %r:%r ]" % (module, exx))
             raise exx
 
         except Exception as exx:
-            log.error("[getUserList][ module %r:%r ]" % (module, exx))
-            log.error("[getUserList] %s" % traceback.format_exc())
+            log.exception("[getUserList][ module %r:%r ]" % (module, exx))
             continue
 
     return users
 
+
+def getUserListIterators(param, search_user):
+    """
+    return a list of iterators for all userid resolvers
+
+    :param param: request params (dict), which might be realm or resolver conf
+    :param search_user: restrict the resolvers to those of the search_user
+    """
+    user_iters = []
+    searchDict = {}
+
+    log.debug("Entering function getUserListIterator")
+
+    searchDict.update(param)
+    if 'realm' in searchDict:
+        del searchDict['realm']
+    if 'resConf' in searchDict:
+        del searchDict['resConf']
+    log.debug("searchDict %r" % searchDict)
+
+    resolverrrs = getResolvers(search_user)
+    for reso in resolverrrs:
+        (package, module, _class, conf) = splitResolver(reso)
+        module = package + "." + module
+
+        if len(search_user.conf) > 0:
+            if conf.lower() != search_user.conf.lower():
+                continue
+
+        # try to load the UserIdResolver Class
+        try:
+            log.debug("Check for resolver class: %r" % reso)
+            y = getResolverObject(reso)
+            log.debug("With this search dictionary: %r " % searchDict)
+
+            if hasattr(y, 'getUserListIterator'):
+                uit = y.getUserListIterator(searchDict, limit_size=False)
+            else:
+                uit = iter(y.getUserList(searchDict))
+
+            user_iters.append((uit, reso))
+
+        except KeyError as exx:
+            log.exception("[ module %r:%r ]" % (module, exx))
+            raise exx
+
+        except Exception as exx:
+            log.exception("[ module %r:%r ]" % (module, exx))
+            continue
+
+    return user_iters
+
+
 def getUserInfo(userid, resolver, resolverC):
     log.debug("[getUserInfo] uid:%r resolver:%r class:%r" %
               (userid, resolver, resolverC))
-                ## [PasswdIdResolver] [IdResolver]
+                # [PasswdIdResolver] [IdResolver]
     userInfo = {}
     module = ""
 
@@ -654,7 +880,7 @@ def getUserInfo(userid, resolver, resolverC):
         return userInfo
 
     try:
-        (package, module, class_, conf) = splitResolver(resolverC)
+        (package, module, _class, _conf) = splitResolver(resolverC)
         module = package + "." + module
 
         y = getResolverObject(resolverC)
@@ -663,10 +889,25 @@ def getUserInfo(userid, resolver, resolverC):
         userInfo = y.getUserInfo(userid)
 
     except Exception as e:
-        log.error("[getUserInfo][ module %r notfound! :%r ]" % (module, e))
+        log.exception("[getUserInfo][ module %r notfound! :%r ]" % (module, e))
 
     return userInfo
 
+
+def getUserDetail(user):
+    '''
+    Returns userinfo of an user
+
+    :param user: the user
+    :returns: the userinfo dict
+    '''
+    (uid, resId, resClass) = getUserId(user)
+    log.debug("got uid %r, ResId %r, Class %r"
+              % (uid, resId, resClass))
+    userinfo = getUserInfo(uid, resId, resClass)
+    return userinfo
+
+
 def getUserPhone(user, phone_type='phone'):
     '''
     Returns the phone numer of a user
@@ -692,65 +933,121 @@ def getUserPhone(user, phone_type='phone'):
                     "type %r." % (uid, resId, resClass, phone_type))
         return ""
 
-def check_user_password(username, realm, password):
+
+def get_authenticated_user(username, realm, password=None,
+                           realm_box=False, authenticate=True,
+                           options=None):
     '''
-    This is a helper function to check the username and password against
-    a userstore.
+    check the username and password against a userstore.
 
-    return
+    remark: the method is called in the context of repoze.who
+            during authentication and during auto_enrollToken/auto_assignToken
 
-      success    --- This is the username of the authenticated user. If unsuccessful,
-                      returns None
+    :param username: the user login name
+    :param realm: the realm, where the user belongs to
+    :param password: the to be checked userstore password
+    :param realm_box: take the information, if realmbox is displayed
+    :parm authenticate: for the selftest, we skip the authentication
+
+    :return: None or authenticated user object
     '''
-    success = None
-    try:
-        log.info("[check_user_password] User %r from realm %r tries to "
-                 "authenticate to selfservice" % (username, realm))
-        if type(username) != unicode:
-            username = username.decode(ENCODING)
-        u = User(username, realm, "")
-        res = getResolversOfUser(u)
-        # Now we know, the resolvers of this user and we can verify the password
-        if (len(res) == 1):
-            (uid, resolver, resolverC) = getUserId(u)
-            log.info("[check_user_password] the user resolves to %r" % uid)
-            log.info("[check_user_password] The username is found within the "
-                     "resolver %r" % resolver)
-            # Authenticate user
-            try:
-                (package, module, class_, conf) = splitResolver(resolverC)
-                module = package + "." + module
-                y = getResolverObject(resolverC)
-            except Exception as e:
-                log.error("[check_user_password] [ module %r notfound! :%r ]"
-                          % (module, e))
-            try:
-                if  y.checkPass(uid, password):
-                    log.debug("[check_user_password] Successfully "
-                              "authenticated user %r." % username)
-                    # try:
-                    #identity = self.add_metadata( environ, identity )
-                    success = username + '@' + realm
-                else:
-                    log.info("[check_user_password] user %r failed "
-                             "to authenticate." % username)
-            except Exception as e:
-                log.error("[check_user_password] Error checking password "
-                          "within module %r:%r" % (module, e))
-                log.error("[check_user_password] %s" % traceback.format_exc())
 
-        elif (len(res) == 0):
-            log.error("[check_user_password] The username %r exists in NO "
-                      "resolver within the realm %r." % (username, realm))
+    log.info("User %r from realm %r tries to authenticate to selfservice"
+             % (username, realm))
+
+    if type(username) != unicode:
+        username = username.decode(ENCODING)
+
+    # ease the handling of options
+    if not options:
+        options = {}
+
+    users = []
+    uid = None
+    resolver = None
+    resolverC = None
+
+    # if we have an realmbox, we take the user as it is
+    # - the realm is always given
+    # - appended realms result in error
+    if realm_box:
+        user = User(username, realm, "")
+        users.append(user)
+
+    # else if no realm box is given
+    #   and realm is not empty:
+    #    - create the user from the values (as we are in auto_assign, etc)
+    #   and the realm is empty! (s. login.mako
+    #    - the user either appends his realm
+    #    - or will get the realm appended
+    #
+    else:
+        if realm:
+            user = User(username, realm, "")
+            users.append(user)
         else:
-            log.error("[check_user_password] The username %r exists in more "
-                      "than one resolver within the realm %r" % (username, realm))
-            log.error(res)
-    except UserError as e:
-        log.error("[check_user_password] Error while trying to verify "
-                  "the username: %r" % e.description)
+            def_realm = options.get('defaultRealm', getDefaultRealm())
+            if def_realm:
+                user = User(username, def_realm, "")
+                users.append(user)
+            if '@' in username:
+                u_name, u_realm = username.rsplit('@', 1)
+                user = User(u_name, u_realm, "")
+                users.append(user)
 
-    return success
+    identified_users = []
+    for user in users:
+        username = user.login
+        realm = user.realm
+        res = getResolversOfUser(user, use_default_realm=False)
+        if (len(res) != 1):
+            if (len(res) == 0):
+                log.info("The username %r exists in NO resolver within the "
+                          "realm %r." % (username, realm))
+            else:
+                log.info("The username %r exists in more than one resolver "
+                          "within the realm %r" % (username, realm))
+            continue
+
+        # we got one resolver, so lets check if user exists
+        (uid, resolver, resolverC) = getUserId(user)
+        identified_users.append((user, uid, resolver, resolverC))
+        log.info("the user resolves to %r" % uid)
+        log.info("The username is found within the resolver %r" % resolver)
+
+    ide_user = len(identified_users)
+    if ide_user != 1:
+        if ide_user > 1:
+            log.info("The username %s could not be identified uniquely" %
+                     username)
+        if ide_user == 0:
+            log.info("The username %s could not be found." % username)
+        return None
+
+    (user, uid, resolver, resolverC) = identified_users[0]
+    if not authenticate:
+        return user
+
+    # Authenticate user
+    auth_user = None
+    try:
+        (package, module, class_, conf) = splitResolver(resolverC)
+        module = package + "." + module
+        y = getResolverObject(resolverC)
+
+        if  y.checkPass(uid, password):
+            log.debug("Successfully authenticated user %r." % username)
+            auth_user = user
+        else:
+            log.info("user %r failed to authenticate." % username)
+
+    except UserError as exx:
+        log.info("failed to verify the username: %s@%s" % (user.login,
+                                                           user.realm))
+
+    if not auth_user:
+        log.error("Error while trying to verify the username: %s" % username)
+
+    return auth_user
 
 #eof###########################################################################
-
