@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -186,6 +186,10 @@ class Token(object):
                 value = linotp.lib.crypt.udecode(value)
             else:
                 value = ""
+        # port of the 2.6. resolver to 2.7
+        if name in ['LinOtpIdResClass']:
+            if value[:len('useridresolveree.')] == 'useridresolveree.':
+                value = "useridresolver.%s" % value[len('useridreseolveree.') - 1:]
 
         return value
 
@@ -995,8 +999,7 @@ class Challenge(object):
             log.debug('save challenge : success')
 
         except Exception as exce:
-            log.error('[save]Error during saving challenge: %r' % exce)
-            log.error("[save] %s" % traceback.format_exc())
+            log.exception('[save]Error during saving challenge: %r' % exce)
 
         return self.transid
 
