@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -194,6 +194,10 @@ XML_PSKC = '''<?xml version="1.0" encoding="UTF-8"?>
 
 class TestImportOTP(TestController):
 
+    def setUp(self):
+        TestController.setUp(self)
+        self.set_config_selftest()
+
     def test_parse_DAT(self):
         '''
         Test to parse of eToken dat file format - import
@@ -452,7 +456,7 @@ sccSignature: MC4CFQDju23MCRqmkWC7Z9sVDB0y0TeEOwIVAOIibmqMFxhPiY7mLlkt5qmRT/xn
         TOKENS = linotp.lib.ImportOTP.parseYubicoCSV(csv)
         print TOKENS
         print len(TOKENS)
-        assert len(TOKENS) == 3
+        assert len(TOKENS) == 5
 
     def test_parse_XML(self):
         '''
@@ -645,5 +649,5 @@ sccSignature: MC4CFQDju23MCRqmkWC7Z9sVDB0y0TeEOwIVAOIibmqMFxhPiY7mLlkt5qmRT/xn
         '''
         response = self.app.post(url(controller='admin', action='loadtokens'), params={'file':csv, 'type':'yubikeycsv'})
         print response
-        assert '"imported": 3' in response
+        assert '"imported": 5' in response
         return
