@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -40,7 +40,6 @@ try:
 except ImportError:
     import simplejson as json
 
-import traceback
 import hashlib
 import base64
 
@@ -159,9 +158,8 @@ class LDAP(object):
                     self.lobj = l_obj
                     break
                 except ldap.LDAPError as e:
-                    log.error("[bind] LDAP error: %r" % e)
+                    log.exception("[bind] LDAP error: %r" % e)
                     log.error("[bind] LDAPURI   : %r" % uri)
-                    log.error("[bind] %s" % traceback.format_exc())
                     raise Exception(e)
 
         return self.lobj
@@ -175,7 +173,7 @@ class LDAP(object):
                 self.lobj.unbind_s()
 
         except ldap.LDAPError as e:
-            log.error("[unbind] LDAP error: %r" % e)
+            log.exception("[unbind] LDAP error: %r" % e)
 
         self.lobj = None
         return ""
@@ -293,6 +291,10 @@ class LDAP(object):
 
 class TestLDAP(TestController):
 
+    def setUp(self):
+        TestController.setUp(self)
+        self.set_config_selftest()
+
     users = [
         {'dn'       : 'cn=Werner Braun,',
         'cn'        : 'Werner Braun',
@@ -482,7 +484,7 @@ class TestLDAP(TestController):
                 incDict = cfgParse.defaults()
                 self.ldapurl = incDict.get('linotp.ldapTestServerIp.'.lower() + hostname, None)
             except Exception as e:
-                log.error('Error parsing include file: %r' % e)
+                log.exception('Error parsing include file: %r' % e)
         else:
                 log.warning('no ldap Test server specified in the ini file!!')
 
@@ -531,14 +533,10 @@ class TestLDAP(TestController):
 
         # Skip test if no config found
         if ldapurl is None or serverIp is None:
-            skip_reason = "No ldap server test url like: ldap://cn=admin,dc=example,dc=com:test123!@192.168.0.2: defined in the *.ini file"
-            if sys.version_info[0:2] >= (2, 7):
-                # skipTest() has the advantage that it is shown in the test summary
-                # but it is only available in Python 2.7
-                self.skipTest(skip_reason)
-            else:
-                log.error("Skipping test 'test_one': " + skip_reason)
-                return
+            skip_reason = "No ldap server test url like: " \
+                "ldap://cn=admin,dc=example,dc=com:test123!@192.168.0.2: " \
+                "defined in the *.ini file"
+            self.skipTest(skip_reason)
 
         parameters = {'enableReplication' : 'true' }
         resp = self.app.get(url(controller='system', action='setConfig'), params=parameters)
@@ -601,8 +599,7 @@ class TestLDAP(TestController):
 
 
         except Exception as e:
-            log.error("%r" % e)
-            log.error("%s" % traceback.format_exc())
+            log.exception("%r" % e)
 
             msg = unicode(e)
             if "Can't contact LDAP server" in msg:
@@ -688,8 +685,7 @@ class TestLDAP(TestController):
 
 
         except Exception as e:
-            log.error(e)
-            log.error("%s" % traceback.format_exc())
+            log.exception(e)
             raise Exception(e)
 
         finally:
