@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -28,7 +28,6 @@
 """
 """
 
-
 import logging
 from linotp.tests import TestController, url
 import copy
@@ -38,14 +37,13 @@ try:
 except ImportError:
     import simplejson as json
 
-
-
 log = logging.getLogger(__name__)
 
 from sqlalchemy.engine import create_engine
 import sqlalchemy
 q = '"'
 
+
 class SQLUser(object):
 
     def __init__(self, connect='sqlite:///:memory:'):
@@ -59,36 +57,61 @@ class SQLUser(object):
             connection = self.engine.connect()
             self.sqlurl = self.engine.url
             if self.sqlurl.drivername == 'mysql':
-                self.userTable = "%s.%s" % (self.sqlurl.database, self.tableName)
+                self.userTable = ("%s.%s"
+                                  % (self.sqlurl.database, self.tableName))
                 self.usercol = 'user'
 
-
-
         except Exception as e:
             print "%r" % e
         self.connection = connection
 
-        umap = { "userid"   : "id",
-                "username"  : "user",
-                "phone"     : "telephonenumber",
-                "mobile"    : "mobile",
-                "email"     : "mail",
-                "surname"   : "sn",
-                "givenname" : "givenname" ,
-                "password"  : "password",
-                "salt"      : "salt" }
-
-        #connect = '''postgres://otpd:linotp2d@localhost/otpdb ''
-
+        umap = {"userid": "id",
+                "username": "user",
+                "phone": "telephonenumber",
+                "mobile": "mobile",
+                "email": "mail",
+                "surname": "sn",
+                "givenname": "givenname",
+                "password": "password",
+                "salt": "salt"}
 
         self.resolverDef = {
-            'Table'     : self.tableName,
-            'Connect'   : connect,
-            'Map'       : json.dumps(umap),
+            'Table': self.tableName,
+            'Map': json.dumps(umap),
          }
 
+        # extend the dict with userid resolver attributes from the connect
+        conn_dict = self._parse_connection(connect)
+        self.resolverDef.update(conn_dict)
+
         return
 
+    def _parse_connection(self, connect):
+        """
+        analyse the sql connection string and transform this to a dict
+        that can be used as an input for an sqluserid resolver
+
+         connect = postgres://otpd:linotp2d@localhost/otpdb
+
+        """
+
+        dbdrive_port, _sep, rest = connect.partition('//')
+        dbdrive, _sep, port = dbdrive_port.partition(':')
+        user_pass, _sep, host_db = rest.partition('@')
+        user, _sep, passw = user_pass.partition(':')
+        host, _sep, db = host_db.partition('/')
+
+        conn = {
+             'Database': db,
+             'Driver': dbdrive,
+             'Server': host,
+             'User': user,
+             'Password': passw,
+             'type': 'sqlresolver',
+            }
+        if port:
+            conn['Port'] = port
+        return conn
 
     def getResolverDefinition(self):
         return self.resolverDef
@@ -118,19 +141,23 @@ class SQLUser(object):
         t = sqlalchemy.sql.expression.text(dropStr)
         self.connection.execute(t)
 
-
-    def addUser(self, user, telephonenumber, mobile, sn, givenname, password, salt, id, mail):
+    def addUser(self, user, telephonenumber, mobile, sn, givenname,
+                password, salt, uid, mail):
         intoStr = """
             INSERT INTO %s( %s, telephonenumber, mobile,
             sn, givenname, password, salt, id, mail)
-            VALUES (:user, :telephonenumber, :mobile, :sn, :givenname, :password, :salt, :id, :mail);
+            VALUES (:user, :telephonenumber, :mobile, :sn, :givenname,
+                    :password, :salt, :id, :mail);
             """ % (self.userTable, self.usercol)
         t = sqlalchemy.sql.expression.text(intoStr)
 
-        self.connection.execute(t, user=user, telephonenumber=telephonenumber, mobile=mobile, sn=sn,
-                                givenname=givenname, password=password, salt=salt, id=id, mail=mail)
+        self.connection.execute(t, user=user, telephonenumber=telephonenumber,
+                                mobile=mobile, sn=sn,
+                                givenname=givenname, password=password,
+                                salt=salt, id=uid, mail=mail)
 
-        #execute(sqlalchemy.sql.expression.text("""SELECT COUNT(*) FROM Config WHERE Config.Key = :key"""), key=REPLICATION_CONFIG_KEY)
+        #execute(sqlalchemy.sql.expression.text("""SELECT COUNT(*)
+        # FROM Config WHERE Config.Key = :key"""), key=REPLICATION_CONFIG_KEY)
 
     def query(self):
         selectStr = "select * from %s" % (self.userTable)
@@ -141,43 +168,35 @@ class SQLUser(object):
 
         return res
 
-    def delUsers(self, id=None, username=None):
+    def delUsers(self, uid=None, username=None):
 
         if username is not None:
             delStr = 'DELETE FROM %s  WHERE user=:user;' % (self.userTable)
             t = sqlalchemy.sql.expression.text(delStr)
             self.connection.execute(t, user=username)
 
-        elif type(id) in (str, u''):
+        elif type(uid) in (str, u''):
             delStr = 'DELETE FROM %s  WHERE id=:id;' % (self.userTable)
             t = sqlalchemy.sql.expression.text(delStr)
-            self.connection.execute(t, id=id)
+            self.connection.execute(t, id=uid)
 
-        elif id is None:
+        elif uid is None:
             delStr = 'DELETE FROM %s ;' % (self.userTable)
             t = sqlalchemy.sql.expression.text(delStr)
             self.connection.execute(t)
 
-
-
-
-
     def close(self):
         self.connection.close()
 
-
     def __del__(self):
         self.connection.close()
 
 
-
-
-
-
 class TestOrphandTokens(TestController):
 
     def setUp(self):
         TestController.setUp(self)
+        self.set_config_selftest()
         self.setUpSQL()
 
     def setUpSQL(self):
@@ -185,7 +204,6 @@ class TestOrphandTokens(TestController):
         self.sqlconnect = self.appconf.get('sqlalchemy.url')
         sqlUser = SQLUser(connect=self.sqlconnect)
         self.sqlResolverDef = sqlUser.getResolverDefinition()
-
         return
 
     def addUsers(self, usercount=10):
@@ -200,7 +218,6 @@ class TestOrphandTokens(TestController):
             log.error(" create user table error: %r " % e)
             userAdd.delUsers()
 
-
         for i in range(1, usercount):
             user = 'hey%d' % i
             telephonenumber = '012345-678-%d' % i
@@ -209,42 +226,44 @@ class TestOrphandTokens(TestController):
             givenname = 'kayak%d' % i
             password = 'safr2r32'
             salt = 't123'
-            id = '__%d' % i
+            uid = '__%d' % i
             mail = sn + '.' + givenname + "@example.com"
 
-            userAdd.addUser(user, telephonenumber, mobile, sn, givenname, password, salt, id, mail)
+            userAdd.addUser(user, telephonenumber, mobile, sn, givenname,
+                            password, salt, uid, mail)
 
         u_dict = [{
-            'user' : 'kn_t',
-            'telephonenumber' : '012345-678-99999',
-            'mobile' : '00123-456-99999',
-            'sn' : 'kn_t',
-            'givenname' : 'knöt',
-            'password' : 'safr2r32',
-            'salt' : 't123',
-            'id' : '__9999',
+            'user': 'kn_t',
+            'telephonenumber': '012345-678-99999',
+            'mobile': '00123-456-99999',
+            'sn': 'kn_t',
+            'givenname': 'knöt',
+            'password': 'safr2r32',
+            'salt': 't123',
+            'uid': '__9999',
             },
-            {'user' : 'knöt',
-            'telephonenumber' : '012345-678-99998',
-            'mobile' : '00123-456-99998',
-            'sn' : 'knöt',
-            'givenname' : 'knöt',
-            'password' : 'safr2r32',
-            'salt' : 't123',
-            'id' : '__9998',
+            {'user': 'knöt',
+            'telephonenumber': '012345-678-99998',
+            'mobile': '00123-456-99998',
+            'sn': 'knöt',
+            'givenname': 'knöt',
+            'password': 'safr2r32',
+            'salt': 't123',
+            'uid': '__9998',
             },
-            {'user' : 'kn%t',
-            'telephonenumber' : '012345-678-99997',
-            'mobile' : '00123-456-99997',
-            'sn' : 'kn%t',
-            'givenname' : 'kn%t',
-            'password' : 'safr2r32',
-            'salt' : 't123',
-            'id' : '__9997',
+            {'user': 'kn%t',
+            'telephonenumber': '012345-678-99997',
+            'mobile': '00123-456-99997',
+            'sn': 'kn%t',
+            'givenname': 'kn%t',
+            'password': 'safr2r32',
+            'salt': 't123',
+            'uid': '__9997',
             },
             ]
         for user in u_dict:
-            user['mail'] = user['sn'] + '.' + user['givenname'] + "@example.com"
+            user['mail'] = ("%s.%s@example.com"
+                            % (user['sn'], user['givenname']))
             userAdd.addUser(**user)
 
         resolverDefinition = userAdd.getResolverDefinition()
@@ -252,11 +271,10 @@ class TestOrphandTokens(TestController):
 
         return resolverDefinition
 
-
-    def delUsers(self, id=None, username=None):
+    def delUsers(self, uid=None, username=None):
 
         userAdd = SQLUser(connect=self.sqlconnect)
-        userAdd.delUsers(id=id, username=username)
+        userAdd.delUsers(uid=uid, username=username)
         userAdd.close()
 
     def addSqlResolver(self, name):
@@ -267,63 +285,68 @@ class TestOrphandTokens(TestController):
         parameters['type'] = 'sqlresolver'
         parameters['Limit'] = '20'
 
-
-        resp = self.app.get(url(controller='system', action='setResolver'), params=parameters)
-        assert('"value": true' in resp)
+        resp = self.app.get(url(controller='system', action='setResolver'),
+                            params=parameters)
+        self.assertTrue('"value": true' in resp, resp)
 
         resp = self.app.get(url(controller='system', action='getResolvers'))
-        assert('"resolvername": "%s"' % (name) in resp)
+        self.assertTrue('"resolvername": "%s"' % (name) in resp, resp)
 
         param2 = {'resolver': name
                   }
-        resp = self.app.get(url(controller='system', action='getResolver'), params=param2)
-        assert('"Table": "User2"' in resp)
+        resp = self.app.get(url(controller='system', action='getResolver'),
+                            params=param2)
+        self.assertTrue('"Table": "User2"' in resp, resp)
 
         return
 
     def delSqlResolver(self, name):
+
         parameters = {
-            'resolver'     : name,
+            'resolver': name,
         }
-        resp = self.app.get(url(controller='system', action='delResolver'), params=parameters)
-        assert('"value": true' in resp)
+        resp = self.app.get(url(controller='system', action='delResolver'),
+                            params=parameters)
+        self.assertTrue('"value": true' in resp, resp)
 
         return resp
 
-
     def addSqlRealm(self, realmName, resolverName, defaultRealm=False):
-        parameters = {
-            'realm'     : realmName,
-            'resolvers' :'useridresolver.SQLIdResolver.IdResolver.%s' % (resolverName)
-        }
-        resp = self.app.get(url(controller='system', action='setRealm'), params=parameters)
-        assert('"value": true' in resp)
+        resolver = 'useridresolver.SQLIdResolver.IdResolver.%s' % resolverName
+        parameters = {'resolvers': resolver,
+             'realm': realmName}
 
-        resp = self.app.get(url(controller='system', action='getRealms'))
-        assert('"default": "true"' in resp)
+        resp = self.make_system_request('setRealm', params=parameters)
+        self.assertTrue('"value": true' in resp, resp)
+
+        if defaultRealm:
+            params = {'realm': realmName}
+            resp = self.make_system_request('setDefaultRealm', params=params)
+            self.assertTrue('"value": true' in resp, resp)
         return
 
     def delSqlRealm(self, realmName):
         parameters = {
-            'realm'     : realmName,
+            'realm': realmName,
         }
-        resp = self.app.get(url(controller='system', action='delRealm'), params=parameters)
-        assert('"result": true' in resp)
+        resp = self.app.get(url(controller='system', action='delRealm'),
+                            params=parameters)
+        self.assertTrue('"result": true' in resp, resp)
 
         return resp
 
-
     def getUserList(self, resolver):
 
-        param = {'username' : '*', 'resConf': resolver}
-        response = self.app.get(url(controller='admin', action='userlist'), params=param)
+        param = {'username': '*', 'resConf': resolver}
+        response = self.app.get(url(controller='admin', action='userlist'),
+                                params=param)
         if ("error") in response:
             body = json.loads(response.body)
             result = body.get('result')
             error = result.get('error')
             raise Exception(error.get('message'))
         else:
-            assert ('"status": true,' in response)
+            self.assertTrue('"status": true,' in response, response)
 
         body = json.loads(response.body)
         result = body.get('result')
@@ -333,59 +356,32 @@ class TestOrphandTokens(TestController):
 
     def addToken(self, user):
 
-        param = { 'user': user, 'pin':user, 'serial': 's' + user, 'type':'spass' }
+        param = {'user': user,
+                 'pin': user,
+                 'serial': 's' + user,
+                 'type': 'spass'}
 
-        response = self.app.get(url(controller='admin', action='init'), params=param)
-        assert '"status": true,' in response
+        response = self.app.get(url(controller='admin', action='init'),
+                                params=param)
+        self.assertTrue('"status": true,' in response, response)
 
         return
 
     def authToken(self, user):
 
-        param = { 'user': user, 'pass':user}
-        response = self.app.get(url(controller='validate', action='check'), params=param)
+        param = {'user': user, 'pass': user}
+        response = self.app.get(url(controller='validate', action='check'),
+                                params=param)
         return response
 
     def showTokens(self):
 
         param = None
-        response = self.app.get(url(controller='admin', action='show'), params=param)
-        assert '"status": true,' in response
+        response = self.app.get(url(controller='admin', action='show'),
+                                params=param)
+        self.assertTrue('"status": true,' in response, response)
         return response
 
-
-    def __deleteAllRealms__(self):
-        ## get al realms
-        response = self.app.get(url(controller='system', action='getRealms'))
-        jResponse = json.loads(response.body)
-        result = jResponse.get("result")
-        values = result.get("value", {})
-        for realmId in values:
-            print realmId
-            realmDesc = values.get(realmId)
-            realmName = realmDesc.get("realmname")
-            parameters = {"realm":realmName}
-            resp = self.app.get(url(controller='system', action='delRealm'), params=parameters)
-            assert('"result": true' in resp)
-
-        return
-
-    def __deleteAllResolvers__(self):
-        ##http://127.0.0.1:5001/system/getResolvers
-        response = self.app.get(url(controller='system', action='getResolvers'))
-        jResponse = json.loads(response.body)
-        result = jResponse.get("result")
-        values = result.get("value", {})
-        for realmId in values:
-            print realmId
-            resolvDesc = values.get(realmId)
-            resolvName = resolvDesc.get("resolvername")
-            parameters = {"resolver" : resolvName}
-            resp = self.app.get(url(controller='system', action='delResolver'), params=parameters)
-            assert('"status": true' in resp)
-
-        return
-
     def test_orphandTokens_byUser(self):
         '''
             test an orphand token - where the user is removed in the sql database
@@ -404,9 +400,8 @@ class TestOrphandTokens(TestController):
         '''
         self.setUpSQL()
 
-        self.__deleteAllRealms__()
-        self.__deleteAllResolvers__()
-
+        self.delete_all_realms()
+        self.delete_all_resolvers()
 
         resolverName = 'MySQLResolver'
         realmName = 'sqlrealm'.lower()
@@ -420,17 +415,17 @@ class TestOrphandTokens(TestController):
 
         self.addToken(user)
         ret = self.authToken(user)
-        assert '"value": true' in ret
+        self.assertTrue('"value": true' in ret, ret)
 
         self.delUsers()
         users = self.getUserList(resolverName)
-        assert len(users) == 0
+        self.assertTrue(len(users) == 0, users)
 
         res = self.showTokens()
-        assert "/:no user info:/" in res
+        self.assertTrue("/:no user info:/" in res, res)
 
         ret = self.authToken(user)
-        assert '"value": false' in ret
+        self.assertTrue('"value": false' in ret, ret)
 
         self.delSqlRealm(realmName)
         self.delSqlResolver(resolverName)
@@ -457,9 +452,8 @@ class TestOrphandTokens(TestController):
         '''
         self.setUpSQL()
 
-        self.__deleteAllRealms__()
-        self.__deleteAllResolvers__()
-
+        self.delete_all_realms()
+        self.delete_all_resolvers()
 
         resolverName = 'MySQLResolver'
         realmName = 'sqlrealm'.lower()
@@ -468,16 +462,14 @@ class TestOrphandTokens(TestController):
         self.addSqlResolver(resolverName)
         self.addSqlRealm(realmName, resolverName, defaultRealm=True)
 
-
         users = self.getUserList(resolverName)
-        assert len(users) > 0
+        self.assertTrue(len(users) > 0, users)
 
         user = users[0].get('username')
 
         self.addToken(user)
         ret = self.authToken(user)
-        assert '"value": true' in ret
-
+        self.assertTrue('"value": true' in ret, ret)
 
         self.delSqlRealm(realmName)
         self.delSqlResolver(resolverName)
@@ -488,35 +480,33 @@ class TestOrphandTokens(TestController):
         except Exception as e:
             message = "%r" % e
             log.error(message)
-        assert len(empty_user_list) == 0
+        self.assertTrue(len(empty_user_list) == 0, empty_user_list)
         #assert "invalid resolver class specification" in message
 
         ret = self.authToken(user)
-        assert '"value": false' in ret
+        self.assertTrue('"value": false' in ret, ret)
 
         res = self.showTokens()
-        assert "/:no user info:/" in res
-
-
+        self.assertTrue("/:no user info:/" in res, res)
 
         return
 
     def test_again(self):
-        for i in range(1, 10):
+        for _i in range(1, 3):
             self.test_orphandTokens_byResolver()
             self.test_orphandTokens_byUser()
 
         return
 
-    def test_0000_umlaut_search(self):
+    def test_umlaut_search(self):
         """
         Escaping SQL Resolver: support for wildcards (s. #12135)
         """
 
         self.setUpSQL()
 
-        self.__deleteAllRealms__()
-        self.__deleteAllResolvers__()
+        self.delete_all_realms()
+        self.delete_all_resolvers()
 
         resolverName = 'MySQLResolver'
         realmName = 'sqlrealm'.lower()
@@ -525,15 +515,16 @@ class TestOrphandTokens(TestController):
         self.addSqlResolver(resolverName)
         self.addSqlRealm(realmName, resolverName, defaultRealm=True)
 
-        parameters = {'username':'knöt'}
+        parameters = {'username': 'knöt'}
         response = self.app.get(url(controller='admin', action='userlist'),
                                                             params=parameters)
+
         self.assertTrue('"userid": "__9998"' in response, response)
         self.assertTrue('"userid": "__9997"' not in response, response)
         self.assertTrue('"userid": "__9999"' not in response, response)
 
         ## ignore SQL wildcards
-        parameters = {'username':'kn%t'}
+        parameters = {'username': 'kn%t'}
         response = self.app.get(url(controller='admin', action='userlist'),
                                                             params=parameters)
 
@@ -542,7 +533,7 @@ class TestOrphandTokens(TestController):
         self.assertTrue('"userid": "__9999"' not in response, response)
 
         ## ignore SQL wildcards
-        parameters = {'username':'kn_t'}
+        parameters = {'username': 'kn_t'}
         response = self.app.get(url(controller='admin', action='userlist'),
                                                             params=parameters)
 
@@ -551,7 +542,7 @@ class TestOrphandTokens(TestController):
         self.assertTrue('"userid": "__9999"' in response, response)
 
         ## support LinOTP wildcards
-        parameters = {'username':'kn*t'}
+        parameters = {'username': 'kn*t'}
         response = self.app.get(url(controller='admin', action='userlist'),
                                                             params=parameters)
 
@@ -560,7 +551,7 @@ class TestOrphandTokens(TestController):
         self.assertTrue('"userid": "__9999"' in response, response)
 
         ## support LinOTP wildcards
-        parameters = {'username':'kn.t'}
+        parameters = {'username': 'kn.t'}
         response = self.app.get(url(controller='admin', action='userlist'),
                                                             params=parameters)
 
@@ -569,7 +560,7 @@ class TestOrphandTokens(TestController):
         self.assertTrue('"userid": "__9999"' in response, response)
 
         ## support LinOTP wildcards for other fields
-        parameters = {'userid':'*9*'}
+        parameters = {'userid': '*9*'}
         response = self.app.get(url(controller='admin', action='userlist'),
                                                             params=parameters)
 
