@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -38,6 +38,16 @@ log = logging.getLogger(__name__)
 
 class TestSelfserviceController(TestController):
 
+    def setUp(self):
+        TestController.setUp(self)
+        self.set_config_selftest()
+        self.create_common_resolvers()
+        self.create_common_realms()
+
+    def tearDown(self):
+        self.delete_all_realms()
+        self.delete_all_resolvers()
+        TestController.tearDown(self)
 
     def createPolicy(self, policy):
         response = self.app.get(url(controller='system', action='setPolicy'),
@@ -65,7 +75,7 @@ class TestSelfserviceController(TestController):
         '''
         self.createPolicy("history")
 
-        response = self.app.get(url(controller='selfservice', action='userhistory'),
+        response = self.app.get(url(controller='userservice', action='history'),
                                 params={'selftest_user': 'root@myDefRealm'})
         print response
         assert '"rows": [' in response
@@ -79,14 +89,14 @@ class TestSelfserviceController(TestController):
         '''
         Selfservice: Testing user reset
         '''
-        response = self.app.get(url(controller='selfservice', action='userreset'),
+        response = self.app.get(url(controller='userservice', action='reset'),
                                 params={'selftest_user': 'root@myDefRealm'})
         print response
         assert '"status": false' in response
         assert '"code": -311' in response
 
         self.createPolicy("reset")
-        response = self.app.get(url(controller='selfservice', action='userreset'),
+        response = self.app.get(url(controller='userservice', action='reset'),
                                 params={'selftest_user': 'root@myDefRealm'})
         print response
         assert 'Missing parameter: ' in response
@@ -108,7 +118,7 @@ class TestSelfserviceController(TestController):
             print response
             assert '"value": false' in response
 
-        response = self.app.get(url(controller='selfservice', action='userreset'),
+        response = self.app.get(url(controller='userservice', action='reset'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'reset01'})
         print response
@@ -130,14 +140,14 @@ class TestSelfserviceController(TestController):
         '''
         Selfservice: Testing user resync
         '''
-        response = self.app.get(url(controller='selfservice', action='userresync'),
+        response = self.app.get(url(controller='userservice', action='resync'),
                                 params={'selftest_user': 'root@myDefRealm'})
         print response
         assert '"status": false' in response
         assert '"code": -311' in response
 
         self.createPolicy("resync")
-        response = self.app.get(url(controller='selfservice', action='userresync'),
+        response = self.app.get(url(controller='userservice', action='resync'),
                                 params={'selftest_user': 'root@myDefRealm'})
         print response
         assert 'Missing parameter' in response
@@ -153,7 +163,7 @@ class TestSelfserviceController(TestController):
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='selfservice', action='userresync'),
+        response = self.app.get(url(controller='userservice', action='resync'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'XXXX',
                                         "otp1": "359864",
@@ -162,7 +172,7 @@ class TestSelfserviceController(TestController):
         assert '"status": false' in response
         assert 'no token found!' in response
 
-        response = self.app.get(url(controller='selfservice', action='userresync'),
+        response = self.app.get(url(controller='userservice', action='resync'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'token01',
                                         "otp1": "885497",
@@ -183,7 +193,7 @@ class TestSelfserviceController(TestController):
         Selfservice: setting mOTP PIN
         '''
 
-        response = self.app.get(url(controller='selfservice', action='usersetmpin'),
+        response = self.app.get(url(controller='userservice', action='setmpin'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'XXXX',
                                         'pin': '1234'})
@@ -192,7 +202,7 @@ class TestSelfserviceController(TestController):
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("setMOTPPIN")
-        response = self.app.get(url(controller='selfservice', action='usersetmpin'),
+        response = self.app.get(url(controller='userservice', action='setmpin'),
                                 params={'selftest_user': 'root@myDefRealm'})
         print response
         assert 'Missing parameter: \'pin\'' in response
@@ -209,7 +219,7 @@ class TestSelfserviceController(TestController):
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='selfservice', action='usersetmpin'),
+        response = self.app.get(url(controller='userservice', action='setmpin'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'token01',
                                         'pin': '1234'})
@@ -235,7 +245,7 @@ class TestSelfserviceController(TestController):
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='selfservice', action='usersetpin'),
+        response = self.app.get(url(controller='userservice', action='setpin'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'spass01',
                                         'pin': '1234'})
@@ -244,14 +254,14 @@ class TestSelfserviceController(TestController):
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("setOTPPIN")
-        response = self.app.get(url(controller='selfservice', action='usersetpin'),
+        response = self.app.get(url(controller='userservice', action='setpin'),
                                 params={'selftest_user': 'root@myDefRealm'})
         print response
         assert 'Missing parameter: \'userpin\'' in response
         assert '"code": 905' in response
 
 
-        response = self.app.get(url(controller='selfservice', action='usersetpin'),
+        response = self.app.get(url(controller='userservice', action='setpin'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'spass01',
                                         'userpin': 'secretPin'})
@@ -283,7 +293,7 @@ class TestSelfserviceController(TestController):
         '''
         self.deleteToken('token01')
 
-        response = self.app.get(url(controller='selfservice', action='usergetSerialByOtp'),
+        response = self.app.get(url(controller='userservice', action='getSerialByOtp'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'type': 'hmac',
                                         'otp': '885497'})
@@ -300,7 +310,7 @@ class TestSelfserviceController(TestController):
         assert '"status": true' in response
 
         self.createPolicy("getserial")
-        response = self.app.get(url(controller='selfservice', action='usergetSerialByOtp'),
+        response = self.app.get(url(controller='userservice', action='getSerialByOtp'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'type': 'hmac',
                                         'otp': '459812'})
@@ -315,7 +325,7 @@ class TestSelfserviceController(TestController):
         assert '"value": 1' in response
 
         # NOw the token is found
-        response = self.app.get(url(controller='selfservice', action='usergetSerialByOtp'),
+        response = self.app.get(url(controller='userservice', action='getSerialByOtp'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'type': 'hmac',
                                         'otp': '459812'})
@@ -345,28 +355,28 @@ class TestSelfserviceController(TestController):
         assert '"value": 1' in response
 
         # Now try to assign
-        response = self.app.get(url(controller='selfservice', action='userassign'),
+        response = self.app.get(url(controller='userservice', action='assign'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'token01'})
         print response
         assert '"message": "ERR410: ' in response
 
         self.createPolicy("assign")
-        response = self.app.get(url(controller='selfservice', action='userassign'),
+        response = self.app.get(url(controller='userservice', action='assign'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'token01'})
         print response
         assert '"assign token": true' in response
 
         # unassign
-        response = self.app.get(url(controller='selfservice', action='userunassign'),
+        response = self.app.get(url(controller='userservice', action='unassign'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'token01'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!",' in response
 
         self.createPolicy("unassign")
-        response = self.app.get(url(controller='selfservice', action='userunassign'),
+        response = self.app.get(url(controller='userservice', action='unassign'),
                                 params={'selftest_user': 'root@myDefRealm',
                                         'serial': 'token01'})
         print response
@@ -399,21 +409,22 @@ class TestSelfserviceController(TestController):
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='selfservice', action='userdelete'),
+        response = self.app.get(url(controller='userservice', action='delete'),
                                 params={'serial': 'token01',
                                         'selftest_user': 'root@myDefRealm'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("delete")
-        response = self.app.get(url(controller='selfservice', action='userdelete'),
+        response = self.app.get(url(controller='userservice', action='delete'),
                                 params={'serial': 'token01',
                                         'selftest_user': 'root@myDefRealm'})
         print response
         assert '"delete token": 1' in response
 
         # UI
-        response = self.app.get(url(controller='selfservice', action='delete'))
+        response = self.app.get(url(controller='selfservice', action='delete'),
+                                params={'selftest_user': 'root@myDefRealm'})
         print response
         assert "<div id='deleteform'>" in response
 
@@ -433,14 +444,14 @@ class TestSelfserviceController(TestController):
         assert '"status": true' in response
 
         # disable
-        response = self.app.get(url(controller='selfservice', action='userdisable'),
+        response = self.app.get(url(controller='userservice', action='disable'),
                                 params={'serial': 'token01',
                                         'selftest_user': 'root@myDefRealm'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!",' in response
 
         self.createPolicy("disable")
-        response = self.app.get(url(controller='selfservice', action='userdisable'),
+        response = self.app.get(url(controller='userservice', action='disable'),
                                 params={'serial': 'token01',
                                         'selftest_user': 'root@myDefRealm'})
         print response
@@ -454,14 +465,14 @@ class TestSelfserviceController(TestController):
 
         # now enable again
 
-        response = self.app.get(url(controller='selfservice', action='userenable'),
+        response = self.app.get(url(controller='userservice', action='enable'),
                                 params={'serial': 'token01',
                                         'selftest_user': 'root@myDefRealm'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("enable")
-        response = self.app.get(url(controller='selfservice', action='userenable'),
+        response = self.app.get(url(controller='userservice', action='enable'),
                                 params={'serial': 'token01',
                                         'selftest_user': 'root@myDefRealm'})
         print response
@@ -474,11 +485,13 @@ class TestSelfserviceController(TestController):
         assert '"LinOtp.Isactive": true' in response
 
         # UI
-        response = self.app.get(url(controller='selfservice', action='disable'))
+        response = self.app.get(url(controller='selfservice', action='disable'),
+                                params={'selftest_user': 'root@myDefRealm'})
         print response
         assert "<div id='disableform'>" in response
 
-        response = self.app.get(url(controller='selfservice', action='enable'))
+        response = self.app.get(url(controller='selfservice', action='enable'),
+                                params={'selftest_user': 'root@myDefRealm'})
         print response
         assert "<div id='enableform'>" in response
 
@@ -488,7 +501,7 @@ class TestSelfserviceController(TestController):
         '''
         self.deleteToken('token01')
 
-        response = self.app.get(url(controller='selfservice', action='userinit'),
+        response = self.app.get(url(controller='userservice', action='enroll'),
                                 params={'serial':'token01',
                                         'type': 'hmac',
                                         'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d',
@@ -499,7 +512,7 @@ class TestSelfserviceController(TestController):
 
         self.createPolicy('enrollHMAC')
 
-        response = self.app.get(url(controller='selfservice', action='userinit'),
+        response = self.app.get(url(controller='userservice', action='enroll'),
                                 params={'serial':'token01',
                                         'type': 'hmac',
                                         'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d',
@@ -520,45 +533,43 @@ class TestSelfserviceController(TestController):
         selfservice: testing user webprovision
         '''
         self.deleteToken('token01')
-        response = self.app.get(url(controller='selfservice', action='userwebprovision'),
+        response = self.app.get(url(controller='userservice', action='webprovision'),
                                 params={'serial':'token01',
                                         'type': 'hmac',
                                         'selftest_user': 'root@myDefRealm'
                                         })
-        print response
-        assert '"message": "valid types are \'oathtoken\' and \'googleauthenticator\' and \'googleauthenticator_time\'. You provided hmac",' in response
+        self.assertTrue('"message": "valid types are \'oathtoken\' and \'googleauthenticator\' and \'googleauthenticator_time\'. You provided hmac",' in response,response)
 
-        response = self.app.get(url(controller='selfservice', action='userwebprovision'),
+        response = self.app.get(url(controller='userservice', action='webprovision'),
                                 params={'serial':'token01',
                                         'type': 'googleauthenticator',
                                         'selftest_user': 'root@myDefRealm'
                                         })
-        print response
-        assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
+
+        self.assertTrue('"message": "ERR410: The policy settings do not allow you to issue this request!"' in response,response)
 
         self.createPolicy('webprovisionGOOGLE')
 
-        response = self.app.get(url(controller='selfservice', action='userwebprovision'),
+        response = self.app.get(url(controller='userservice', action='webprovision'),
                                 params={'prefix':'LSGO',
                                         'type': 'googleauthenticator',
                                         'selftest_user': 'root@myDefRealm'
                                         })
-        print response
-        assert '"url": "otpauth://hotp/LSGO' in response
+        self.assertTrue('"url": "otpauth://hotp/LSGO' in response, response)
 
         # test
         response = self.app.get(url(controller='admin', action='show'),
                                 params={'user': 'root'})
-        print response
-        assert '"LinOtp.TokenSerialnumber": "LSGO' in response
-        assert '"LinOtp.Isactive": true' in response
+        self.assertTrue('"LinOtp.TokenSerialnumber": "LSGO' in response,response)
+        self.assertTrue('"LinOtp.Isactive": true' in response, response)
 
         # UI
 
         response = self.app.get(url(controller='selfservice', action='webprovisiongoogletoken'),
                                 params={'selftest_user': 'root@myDefRealm'})
-        print response
-        assert "<div id='googletokenform'>" in response
+        self.assertTrue("googletokenform" in response.body, response)
+
+        return
 
 
     def test_ocra(self):
