@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -230,9 +230,16 @@ class TestTotpController(TestController):
     '''
     def setUp(self):
         TestController.setUp(self)
+        self.set_config_selftest()
+        self.create_common_resolvers()
+        self.create_common_realms()
         self.serials = []
 
-
+    def tearDown(self):
+        self.removeTokens()
+        self.delete_all_realms()
+        self.delete_all_resolvers()
+        TestController.tearDown(self)
 
     def removeTokens(self):
         for serial in self.serials:
@@ -261,8 +268,7 @@ class TestTotpController(TestController):
             try:
                 dt = datetime.datetime.strptime(curTime, tFormat)
             except Exception as e:
-                log.error('[time2float] Error during conversion of datetime: %r' % e)
-                log.error("[time2float] %s" % traceback.format_exc())
+                log.exception('[time2float] Error during conversion of datetime: %r' % e)
                 raise Exception(e)
         else:
             log.error("[time2float] invalid curTime: %s. You need to specify a datetime.datetime" % type(curTime))
@@ -558,10 +564,3 @@ class TestTotpController(TestController):
             self.removeTokens()
 
         return
-
-
-
-    def tearDown(self):
-        self.removeTokens()
-
-        TestController.tearDown(self)
